package somlib.som;

import java.lang.Math;

public class EuclideanMap extends Map
/*
 * the Euclidean Map is an actual implementation of Map, it represents a 'rect'-topology
 */
{

  public EuclideanMap(int XDim, int YDim, int dimensions, long seed, somlib.textrepresentation.Logfile log, byte verbose) throws SOMException {
    super(XDim, YDim, dimensions, seed, log, verbose);

    if(log!=null)
      log.append("Start: EuclideanMap opened.");
  }

  public EuclideanMap(int XDim, int YDim, long seed, somlib.textrepresentation.Logfile log, byte verbose) throws SOMException {
    super(XDim, YDim, seed, log, verbose);

    if(log!=null)
      log.append("Start: EuclideanMap opened.");
  }

  public EuclideanMap(String url_weight_vec, String url_quanterr_map, boolean clearQuant, String url_node_descr, somlib.textrepresentation.Logfile log, byte verbose) throws SOMException {
    super(url_weight_vec, url_quanterr_map, clearQuant, url_node_descr, log, verbose);

    if(log!=null)
      log.append("Start: EuclideanMap opened.");
  }

  public Coordinate winnerAt(Vector vector) throws SOMException
  /*
   * the most similar Node-Vector to @vector is returned;
   * the similarity is measured by the euclidean distance
   */
  {
    float dist2, distance=Integer.MAX_VALUE;
    Coordinate wins=new Coordinate();

    for(int x=0; x<XDim; x++) {
      for(int y=0; y<YDim; y++) {
        try {
          if ((dist2=Vector.sub(card[x][y].returnVector(), vector).makeDimPos().euclLength()) < distance) {
            distance=dist2;
            wins.x=x;
            wins.y=y;
          }
        } catch(SOMException e) {
          throw new SOMException("EuclideanMap.winnerAt: "+e.getMessage());
        }
      }
    }
    if (verbose > 2) System.out.println("        Map: located winner: "+card[wins.x][wins.y].returnVector().returnAsString()+" to vector: "+vector.returnAsString());

    return wins;
  }

  public Coordinate map(Vector vector) throws SOMException
  /*
   * same as winnerAt(Vector), but it adapts the quantization error
   */
  {
    Coordinate winner=winnerAt(vector);
    Vector quantVec=Vector.sub(vectorAt(winner), vector).makeDimPos();
    float quantVal=quantVec.euclLength();

    if (verbose > 1) System.out.println("        Mapped Vector named "+vector.getLabel()+" to "+winner.x+"/"+winner.y);
    mappingFile.addItem(new somlib.som.descriptions.MappingItem(winner.x, winner.y, vector.getLabel(), quantVal) );

    quanterror+=quantVal;
    try {
      card[winner.x][winner.y].adaptQuant(Vector.add(card[winner.x][winner.y].returnQuantVector(), quantVec), quantVal);
    } catch(SOMException e) {
      throw new SOMException("EuclideanMap.map: "+e.getMessage());
    }
    card[winner.x][winner.y].addToVecsMapped(vector.getLabel(), quantVal);

    return winner;
  }

  public float hasDistanceToWinner(Vector anyVec) throws SOMException
  /*
   * the closest euclidean distance of the Vector @anyVec to a node-vector on the map is returned
   */
  {
    return (Vector.sub(winner(anyVec), anyVec)).makeDimPos().euclLength();
  }

  public float dist_ci(Coordinate winner, int x, int y)
    /*
      calculates the distance between two points - one given in coordinates, the other in x/y;
      needed to retrieve the distance of the winner to its neighbors taking the map-topology into account
    */
  {
    return (float)Math.sqrt( (winner.x-x)*(winner.x-x)+(winner.y-y)*(winner.y-y) );
  }

}
