package somlib.som;

import java.io.*;

abstract class Map
/*
 * this class implements the requirements, any Map - no matter which topology - has to fulfill
 */
{

  somlib.textrepresentation.Logfile log = null;
  byte verbose = 2;

  int XDim;                                         // the dimensions of the map in x -
  int YDim;                                         // and in y - direction
  int vectorDimension = -1;             // the dimension of the vectors
  String type = "rect";                      // the standard topology
  long seed=-1;                                  // the seed for initializing the nodes
  Node[][] card;                                 // the map is an array of Nodes
  float quanterror=0.0f;                 // the quantisation error of the whole map
  somlib.som.descriptions.Mapping mappingFile = null;
  somlib.som.descriptions.WeightVector wv= null;
  somlib.som.descriptions.Err err = null;
  somlib.som.descriptions.Node node = null;

  public Map(int XDim, int YDim, long seed, somlib.textrepresentation.Logfile log, byte verbose) throws SOMException {
    if(seed < 0) this.seed = (new java.util.Date()).getTime();
    else this.seed=seed;
    this.log=log;
    this.verbose=verbose;
    java.util.Random rnd = new java.util.Random(seed);
    if( ((this.XDim=XDim)<=0) || ((this.YDim=YDim)<=0) )
      throw new SOMException("Map: wrong Dimensions.");

    card=new Node[XDim][YDim];
    somlib.som.descriptions.LabelFactory labeler = new somlib.som.descriptions.LabelFactory();
    for(int x=0; x<XDim; x++) {
      for(int y=0; y<YDim; y++) {
        card[x][y]=new Node(vectorDimension, rnd, labeler.label(x,y));
      }
    }
    resetQuantError();
    initMappingFile();
    if (verbose > 0) System.out.println("  Map created.");
  }

  public Map(int XDim, int YDim, int dimensions, long seed, somlib.textrepresentation.Logfile log, byte verbose) throws SOMException {
    if(seed < 0) this.seed = (new java.util.Date()).getTime();
    else this.seed=seed;
    this.log=log;
    this.verbose=verbose;
    this.vectorDimension=dimensions;
    java.util.Random rnd = new java.util.Random(seed);
    if( ((this.XDim=XDim)<=0) || ((this.YDim=YDim)<=0) )
      throw new SOMException("Map: wrong Dimensions.");

    card=new Node[XDim][YDim];
    somlib.som.descriptions.LabelFactory labeler = new somlib.som.descriptions.LabelFactory();
    for(int x=0; x<XDim; x++) {
      for(int y=0; y<YDim; y++) {
        card[x][y]=new Node(dimensions, rnd, labeler.label(x,y));
      }
    }
    resetQuantError();
    initMappingFile();
    if (verbose > 0) System.out.println("  Map created.");
  }

  public Map(String url_weight_vec, String url_quanterr_map, boolean clearQuant, String url_node_descr, somlib.textrepresentation.Logfile log, byte verbose) throws SOMException {
    this.log=log;
    this.verbose=verbose;

    wv = new somlib.som.descriptions.WeightVector(url_weight_vec);
    node = new somlib.som.descriptions.Node(url_node_descr);
    if(url_quanterr_map!=null)
      err = new somlib.som.descriptions.Err(url_quanterr_map);
    else
      clearQuant=true;

    this.XDim = wv.XDim.parameter;
    this.YDim = wv.YDim.parameter;
    if(err!=null && (XDim!=err.XDim.parameter || YDim!=err.YDim.parameter))
      throw new SOMException("Map.<init>: The dimensions of the description-files don't match!");
    this.vectorDimension = wv.vec_dim.parameter;
    this.type = wv.type.parameter;
    card=new Node[XDim][YDim];

    for(int x=0; x<XDim; x++) {
      for(int y=0; y<YDim; y++) {
        this.card[x][y]=new somlib.som.Node(new Vector(wv.vecs[x][y], wv.labels[x][y]), node, x, y);
        if(!clearQuant) {
          somlib.som.descriptions.NodeItem ndi=node.getNodeAt(x,y);
          if(ndi!=null)
            this.card[x][y].adaptQuant(new Vector((float[])err.vecs[x][y], (String)err.labels[x][y]), ndi.quanterror_unit.parameter);
          else System.err.println("  Couldn't find Unit "+x+"/"+"y");
         }
      }
    }
    if(clearQuant)
      resetQuantError();

    initMappingFile();
    if (verbose > 0) System.out.println("  Map created.");
  }

  public Vector vectorAt(Coordinate pos)
  /*
   * returns the Vector at the spezified coordinates
   */
  {
    return new Vector(card[pos.x][pos.y].returnVector());
  }

  public void adapt(Coordinate position, Vector toAdapt)
  /*
   * the Vector at the spezified @position is adapted to the new value of @toAdapt
   */
  {
    if (verbose > 2) System.out.println("      Map: adapting Vector at x: "+position.x+" y: "+position.y);
    card [position.x] [position.y].adaptVector(toAdapt);
  }

  public void resetQuantError()
  /*
   * the quantization error of the whole map and each node is set back
   */
  {
    if (verbose > 1) System.out.println("      Map: resetting Quantization Error");
    quanterror=0.0f;
    for(int x=0; x<XDim; x++) {
      for(int y=0; y<YDim; y++) {
        card[x][y].resetQuantError();
      }
    }
  }

  public void  initMappingFile()
  /*
   * the File containing information about previous Mappings is cleared
   */
  {
    mappingFile = new somlib.som.descriptions.Mapping(XDim, YDim, vectorDimension);
  }

  public float getTotalQuantizationError() {
    return quanterror;
  }

  public void setTopology(String topology) {
    this.type=topology;
  }

  public String getTopology() {
    return type;
  }

  public void storeToWgtFile(String filename) throws SOMException
  /*
   * the vector-value of each node is written to a SOM Weight Vector File named @filename
   */
  {
    float[][][] vecs = new float[XDim][YDim][vectorDimension];
    String[][] labels = new String[XDim][YDim];

    if (verbose > 0) System.out.println("      Map: storing SOM Weigth Vectors");
    for(int x=0; x<XDim; x++) {
      for(int y=0; y<YDim; y++) {
        vecs[x][y]=card[x][y].vectorValue();
        labels[x][y]=card[x][y].giveLabel();
      }
    }
    try {
      if(this.wv != null)
        this.wv.update(vecs, labels).writeDescr(filename);
      else
        (new somlib.som.descriptions.WeightVector(type, XDim, YDim, vectorDimension, vecs, labels)).writeDescr(filename);
    } catch(somlib.som.descriptions.DescriptionException e) {
      throw new SOMException("Map.storeToWgtFile: "+e.getMessage());
    }
  }

  public void storeQuantToErrFile(String filename) throws SOMException
  /*
   * the quantisation error of each node is written to a SOM Quantization Error Map named @filename
   */
  {
    float[][][] vecs = new float[XDim][YDim][vectorDimension];
    String[][] labels = new String[XDim][YDim];

    if (verbose > 0) System.out.println("      Map: writing SOM Quantization Error Map");
    for(int x=0; x<XDim; x++) {
      for(int y=0; y<YDim; y++) {
        vecs[x][y]=card[x][y].quantVectorValue();
        labels[x][y]=card[x][y].giveLabel();
      }
    }
    try {
      if(this.err!=null)
        this.err.update(vecs, labels).writeDescr(filename);
      else
        ( new somlib.som.descriptions.Err("qerr_"+type, XDim, YDim, vectorDimension, vecs, labels)).writeDescr(filename);
    } catch(somlib.som.descriptions.DescriptionException e) {
      throw new SOMException("Map.storeToErrFile: "+e.getMessage());
    }
  }

  public void storeNodes(String filename) throws SOMException
  /*
   * the vector-value of each node is written to SOM Node Descriptions
   * named @filename appended by the coordinates of the very Node
   */
  {
    somlib.som.descriptions.NodeItem ndi;
    if(node==null)
      node = new somlib.som.descriptions.Node(type, XDim, YDim);

    if (verbose > 0) System.out.println("      Map: storing SOM Unit Descriptions");
    for(int y=0; y<YDim; y++) {
      for(int x=0; x<XDim; x++) {
        if( (ndi = node.getNodeAt(x, y)) == null) {
          ndi = createNode(new somlib.som.descriptions.NodeItem(), x, y);
          node.addItem(ndi);
        } else
          createNode(ndi, x, y);
      }
    }
    try {
      node.writeDescr(filename);
    } catch(somlib.som.descriptions.DescriptionException e) {
      throw new SOMException("Map.storeNodes: "+e.getMessage());
    }
  }

  public somlib.som.descriptions.NodeItem createNode(somlib.som.descriptions.NodeItem ndi, int x, int y) throws somlib.som.descriptions.DescriptionException, SOMException {
    ndi.pos_x.parameter=x;
    ndi.pos_y.parameter=y;
    ndi.unit_id.parameter  = card[x][y].giveLabel();
    ndi.quanterror_unit.parameter = card[x][y].quantisationError();
    if((x==XDim-1) && (y==YDim-1)) {    //rechts unten
      ndi.umat_right.parameter = -1;
      ndi.umat_down_right.parameter = -1;
      ndi.umat_down.parameter = -1;
      ndi.umat_down_left.parameter = -1;
      ndi.umat_unit.parameter = -1;
    } else if(y==YDim-1) {                 //unten
      ndi.umat_right.parameter = (Vector.sub(card[x][y].returnVector(), card[x+1][y].returnVector())).euclLength();
      ndi.umat_down_right.parameter = -1;
      ndi.umat_down.parameter = -1;
      ndi.umat_down_left.parameter = -1;
      ndi.umat_unit.parameter = ndi.umat_right.parameter;
    } else if((x==0) && (x==XDim-1)) {                            //links ... XDim=1
      ndi.umat_right.parameter = -1;
      ndi.umat_down_right.parameter = -1;
      ndi.umat_down.parameter = -1;
      ndi.umat_down.parameter = (Vector.sub(card[x][y].returnVector(), card[x][y+1].returnVector())).euclLength();
      ndi.umat_unit.parameter = ndi.umat_down.parameter;
    } else if(x==0) {                            //links
      ndi.umat_right.parameter = (Vector.sub(card[x][y].returnVector(), card[x+1][y].returnVector())).euclLength();
      ndi.umat_down_right.parameter = (Vector.sub(card[x][y].returnVector(), card[x+1][y+1].returnVector())).euclLength();
      ndi.umat_down.parameter = (Vector.sub(card[x][y].returnVector(), card[x][y+1].returnVector())).euclLength();
      ndi.umat_down_left.parameter = -1;
      ndi.umat_unit.parameter = (ndi.umat_right.parameter + ndi.umat_down_right.parameter + ndi.umat_down.parameter)/3;
    } else if(x==XDim-1) {                //rechts
      ndi.umat_right.parameter = -1;
      ndi.umat_down_right.parameter = -1;
      ndi.umat_down.parameter = (Vector.sub(card[x][y].returnVector(), card[x][y+1].returnVector())).euclLength();
      ndi.umat_down_left.parameter = (Vector.sub(card[x][y].returnVector(), card[x-1][y+1].returnVector())).euclLength();
      ndi.umat_unit.parameter = (ndi.umat_down.parameter + ndi.umat_down_left.parameter)/2;
    } else {                                           //normal
      ndi.umat_right.parameter = (Vector.sub(card[x][y].returnVector(), card[x+1][y].returnVector())).euclLength();
      ndi.umat_down_right.parameter = (Vector.sub(card[x][y].returnVector(), card[x+1][y+1].returnVector())).euclLength();
      ndi.umat_down.parameter = (Vector.sub(card[x][y].returnVector(), card[x][y+1].returnVector())).euclLength();
      ndi.umat_down_left.parameter = (Vector.sub(card[x][y].returnVector(), card[x-1][y+1].returnVector())).euclLength();
      ndi.umat_unit.parameter = (ndi.umat_right.parameter + ndi.umat_down_right.parameter + ndi.umat_down.parameter + ndi.umat_down_left.parameter)/4;
    }
    ndi.nr_vec_mapped.parameter = card[x][y].nrVecsMapped();
    ndi.mapped_vecs.parameter = card[x][y].vecsMapped();
    if(ndi.nr_vec_mapped.parameter > 0)
      ndi.mapped_vecs_dist.read((Float[])card[x][y].mappedVecsDist().toArray(new Float[0]));
//          ndi.unit_labels = ;  written by labelling program
    ndi.description.parameter  = card[x][y].description;

    return ndi;
  }

  public void writeMappingFile(String filename) throws SOMException {
    if (verbose > 0) System.out.println("      Map: writing Mapping File");
    mappingFile.writeDescr(filename);
  }

  public Vector winner(Vector vector) throws SOMException {
    return new Vector(vectorAt(winnerAt(vector)));
  }


    /* the following methods need to be implemented by a specific node topology */
  public abstract Coordinate winnerAt(Vector vector) throws SOMException;
  public abstract Coordinate map(Vector vector) throws SOMException;
  public abstract float hasDistanceToWinner(Vector anyVec) throws SOMException;
  public abstract float dist_ci(Coordinate winner, int x, int y);

}
