package somlib.som;

import java.io.*;

public class Node
/*
 * this structure represents the nodes, a SOMap consists of
 */
{

  private Vector vector;                                                       // the node's value
  private Vector quantisationError;                                   // the total quantisation error-Vector of the node
  private float quantisationErrorVal;                                   // the total quantisation error of the node
  private int nr_mapped_vecs = 0;                                    // the total number of Vectors mapped and
  private java.lang.StringBuffer mapped_vecs = null;   // the labels of all Vectors mapped on the node
  private java.util.Vector mapped_vecs_dist = null;      // distances by which vectors are mapped onto the unit
  public String description = null;                                     //any notes concerning the specific node

  public Node(Vector vector) {
    this.vector=new Vector(vector);
    this.quantisationError=new Vector(vector.dims, "QuantError: "+giveLabel());
    resetQuantError();
  }

  public Node(int dims, java.util.Random rnd, String label) {
    this.vector=new Vector(dims, rnd, label);
    this.quantisationError=new Vector(vector.dims, "QuantError: "+giveLabel());
    resetQuantError();
  }

  public Node(Vector vector, somlib.som.descriptions.Node node, int x, int y) throws SOMException {
    somlib.som.descriptions.NodeItem ndi = null;
    this.vector=new Vector(vector);
    this.quantisationError=new Vector(vector.dims, "QuantError: "+giveLabel());
    resetQuantError();

    if( (ndi = node.getNodeAt(x, y)) == null)
      throw new SOMException("Couldn't find unit "+x+"/"+y);
    if(ndi.nr_vec_mapped.parameter > 0)
      this.nr_mapped_vecs = ndi.nr_vec_mapped.parameter;
    this.mapped_vecs.append(ndi.mapped_vecs.parameter);
    this.mapped_vecs_dist = ndi.mappedVecsDist();
    this.description=ndi.description.parameter;
  }

  public void resetQuantError() {
    quantisationError.reset();
    quantisationErrorVal=0;
    nr_mapped_vecs=0;
    mapped_vecs=new java.lang.StringBuffer();
    mapped_vecs_dist = new java.util.Vector();
  }

  public void adaptVector(Vector toAdapt)
  /*
   * the node's vector value is newly initialized by the value of @toAdapt
   */
  {
    vector.setTo(new Vector(toAdapt));
  }

  public Vector returnVector() {
    return new Vector(vector);
  }

  public void adaptQuant(Vector toAdapt, float value)
  /*
   * the node's quantisation error vector is newly initialized by the value of @toAdapt
   * @value is added to the Units quantisation error
   */
  {
    quantisationError.setTo(new Vector(toAdapt));
    quantisationErrorVal+=value;
  }

  public Vector returnQuantVector()
  /*
   * the whole quantisation error vector is returned
   */
  {
    return new Vector(quantisationError);
  }

  public float[] quantVectorValue()
  /*
   * the values of the quantisation error vector are returned
   */
  {
    return quantisationError.value();
  }

  public float quantisationError()
  /*
   * the total quantisation error of the node is returned
   */
  {
    if(nr_mapped_vecs<=0)
      return -1;

    return quantisationErrorVal;
  }

  public float[] vectorValue() {
    return vector.value();
  }

  public void addToVecsMapped(String label, float qe)
  /*
   * another vector named @label with the Quantisation error @qe is mapped
   */
  {
    nr_mapped_vecs++;
    mapped_vecs.append(label+" ");
    mapped_vecs_dist.add(new Float(qe));
  }

  public String vecsMapped()
  /*
   * the names of all the vectors mapped on the node are returned
   */
  {
    if(mapped_vecs.length()<=0)
      return null;

    return mapped_vecs.substring(0,mapped_vecs.length()-1);
  }

  public int nrVecsMapped()
  /*
   * the number of all the vectors mapped on the node are returned
   */
  {
    if(nr_mapped_vecs<=0)
      return -1;

    return nr_mapped_vecs;
  }

  public java.util.Vector mappedVecsDist()
  /*
   * the distances by which vectors are mapped onto the unit are returned
   */
  {
    if(nr_mapped_vecs<=0)
      return null;

    return new java.util.Vector(mapped_vecs_dist);
  }

  public String giveLabel()
  /*
   * the node's name
   */
  {
    return vector.getLabel();
  }

}
