package somlib.som.descriptions;

import java.io.*;
import java.util.*;

public class LibViewer {
/*
 * this class describes the information needed by the LibViewer
 */

    /** Parameters:  **/
  public String serv_url=new String("bergman.ifs.tuwien.ac.at");
  public String serv_port=new String("6666");
  public String serv_descr=new String("SOMLib");
  public String serv_data=new String("file");
  public String serv_but1=new String("load");
  public String serv_but2=new String();
  public String serv_but1txt=new String("Load File");
  public String serv_but2txt=new String();
  public String serv_reserved=new String("void");

  public Vector items = null;


  public LibViewer() {
    items = new Vector();
  }

  public LibViewer(String filename) throws DescriptionException {
    File file = new File(filename);
    if((!file.exists()) || (!file.canRead()))
      throw new DescriptionException("Can't read file: "+file.toString());

    items = new Vector();
    readDescr(file);
  }

  public void addItem(LibItem libi) {
    items.add(libi);
  }

  public void writeDescr(String filename, String node_descr, int xdim, int ydim) throws DescriptionException
  /*
   * the LibViewer-Description is written to @filename, if node_descr is not null,
   * a partition into shelves similar to the topology of the SOMap is printed
   */
  {
    File file = new File(filename);

    try {
      BufferedWriter out = new BufferedWriter(new FileWriter(file));

      out.write("$ITEMCOUNT "+ items.size()); out.newLine();
      if(serv_url != null)
        out.write("$SERV_URL "+serv_url); out.newLine();
      if(serv_port != null)
        out.write("$SERV_PORT "+serv_port); out.newLine();
      if(serv_descr != null)
        out.write("$SERV_DESCR "+serv_descr); out.newLine();
      if(serv_data != null)
        out.write("$SERV_DATA "+serv_data); out.newLine();
      if(serv_but1 != null)
        out.write("$SERV_BUT1 "+serv_but1); out.newLine();
      if(serv_but2 != null)
        out.write("$SERV_BUT2 "+serv_but2); out.newLine();
      if(serv_but1txt != null)
        out.write("$SERV_BUT1TXT "+serv_but1txt); out.newLine();
      if(serv_but2txt != null)
        out.write("$SERV_BUT2TXT "+serv_but2txt); out.newLine();
      if(serv_reserved != null)
        out.write("$SERV_RESERVED "+serv_reserved); out.newLine();

      if(node_descr!=null) {
        out.write("#-------------------------------"); out.newLine();
        writeShelves(out, node_descr, xdim, ydim);
      }

      for (Enumeration enum = items.elements(); enum.hasMoreElements(); ) {
        out.write("#-------------------------------"); out.newLine();
        ((LibItem)enum.nextElement()).writeDescr(out);
        out.flush();
      }

      out.close();
    } catch(IOException e) {
      throw new DescriptionException("File communication failure: "+e.getMessage());
    }
  }

   private void writeShelves(BufferedWriter out, String node_descr, int xdim, int ydim) throws DescriptionException, IOException
  /*
   * the number of shelves and their description is written to @out
   */
  {
      somlib.som.descriptions.Node nd = new somlib.som.descriptions.Node(node_descr);
      NodeItem ndi;
      out.write("$SHELVE_ROWS "+ydim); out.newLine();
      out.write("$SHELVE_COLUMNS "+xdim); out.newLine();

      for(int y=0; y<ydim; y++) {
        for(int x=0; x<xdim; x++) {
          out.write("#Unit "+y+"/"+x+"---------------"); out.newLine();
          if( (ndi = nd.getNodeAt(x, y))!=null) {
            out.write("$SHELVE_LABEL ");
            if(ndi.unit_id.parameter != null) out.write(ndi.unit_id.parameter+" -");
            if(ndi.unit_labels.parameter!=null) ndi.unit_labels.writeTo(out);
            out.newLine();
            if(ndi.description.parameter!=null) {
              out.write("$SHELVE_DESCR");
              out.write(ndi.description.parameter);
              out.newLine();
            }// else {out.write("$SHELVE_DESCR unlabelled"); out.newLine(); von oben runter}
          }
        }
      }
   }


  private void readDescr(File source) throws DescriptionException {
    if(!source.exists() || !source.canRead())
      throw new DescriptionException("Couldn't find file: "+source.toString());

    String str, teil;
    int itemcount=0;
    LibItem libi = new LibItem();
    try {
      BufferedReader in = new BufferedReader(new FileReader(source));
      while ((str = in.readLine()) != null) {
        if(str.startsWith("$")) {
          teil=str.substring(1,str.indexOf(" "));

          if(teil.equals("SERV_URL"))
            serv_url=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("SERV_PORT"))
            serv_port=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("SERV_DESCR"))
            serv_descr=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("SERV_DATA"))
            serv_data=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("SERV_BUT1"))
            serv_but1=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("SERV_BUT2"))
            serv_but2=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("SERV_BUT1TXT"))
            serv_but1txt=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("SERV_BUT2TXT"))
            serv_but2txt=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("SERV_RESERVED"))
            serv_reserved=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("ITEMCOUNT"))
            itemcount=Integer.parseInt(str.substring(str.indexOf(" ")+1));
          else {

            do {
              if(str.startsWith("$")) {
                teil=str.substring(1,str.indexOf(" "));

                if(teil.equals("TITLE")) {
                  if(libi.title == null)
                    libi.title=str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("WIDTH")) {
                  if(libi.width == -1)
                    libi.width=Integer.parseInt(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("HEIGHT")) {
                  if(libi.height == -1)
                    libi.height=Integer.parseInt(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("CREATOR")) {
                  if(libi.creator == null)
                    libi.creator =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("SUBJECT")) {
                  if(libi.subject == null)
                    libi.subject =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("DESCRIPTION")) {
                  if(libi.description == null)
                    libi.description =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("PUBLISHER")) {
                  if(libi.publisher == null)
                    libi.publisher =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("CONTRIBUTOR")) {
                  if(libi.contributor == null)
                    libi.contributor =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("DATE")) {
                  if(libi.date == null)
                    libi.date =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("TYPE")) {
                  if(libi.type == null)
                    libi.type =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("FORMAT")) {
                  if(libi.format == null)
                    libi.format =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("IDENTIFIER")) {
                  if(libi.identifier == null)
                    libi.identifier =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("RELATION")) {
                  if(libi.relation == null)
                    libi.relation =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("SIZE_BYTE")) {
                  if(libi.size_byte == null)
                    libi.size_byte =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("IMAGE_LOGO")) {
                  if(libi.image_logo == null)
                    libi.image_logo =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("TEMPLATE")) {
                  if(libi.template == null)
                    libi.template =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("WELLTHUMBED")) {
                  if(libi.wellthumbed == null)
                    libi.wellthumbed =str.substring(str.indexOf(" ")+1);
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("DUST_LEVEL")) {
                  if(libi.dust_level == -1)
                    libi.dust_level =Integer.parseInt(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("COLOR_FRONT")) {
                  if(libi.color_front == null)
                    libi.color_front = new RGB(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("COLOR_TOP")) {
                  if(libi.color_top == null)
                    libi.color_top = new RGB(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("COLOR_SPINE")) {
                  if(libi.color_spine == null)
                    libi.color_spine = new RGB(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("COLOR_TITLE")) {
                  if(libi.color_title == null)
                    libi.color_title = new RGB(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("COLOR_CREATOR")) {
                  if(libi.color_creator == null)
                    libi.color_creator = new RGB(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("HIGHLIGHTING")) {
                  if(libi.highlighting == -1)
                    libi.highlighting =Integer.parseInt(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("XPOS")) {
                  if(libi.XPos == -1)
                    libi.XPos =Integer.parseInt(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("YPOS")) {
                  if(libi.YPos== -1)
                    libi.YPos=Integer.parseInt(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else if(teil.equals("ZPOS")) {
                  if(libi.ZPos == -1)
                    libi.ZPos =Float.parseFloat(str.substring(str.indexOf(" ")+1));
                  else {
                    items.add(libi); libi = new LibItem(); continue;
                  }
                } else
                  throw new DescriptionException("Wrong formatted file.");
              }
              str = in.readLine();
            } while(str != null);
            items.add(libi);

          }

        }
        if(itemcount != items.size())
          throw new DescriptionException("Wrong $ITEMCOUNT given.");
      }
      in.close();
    } catch(IOException e) {
      throw new DescriptionException("Failure communicating with file: "+e.getMessage());
    }
  }

}
