package somlib.som.descriptions;

import java.io.*;

public class Map {
/*
 * this class handles file communication with a  SOM Map Description File
 */

  /** Parameters:  **/
  private java.util.Vector unknown = null;
  private java.lang.StringBuffer comment = new java.lang.StringBuffer();
       //mandatory
  public StringParameter type=new StringParameter();
  public IntegerParameter XDim=new IntegerParameter();
  public IntegerParameter YDim=new IntegerParameter();
  public IntegerParameter vec_dim=new IntegerParameter();

        //optional
  public String filename=null;
  public StringParameter storage_date=new StringParameter();
  public StringParameter storage_time=new StringParameter();
  public IntegerParameter training_time=new IntegerParameter();
  public StringParameter learnrate_type=new StringParameter();
  public FloatParameter learnrate_init=new FloatParameter();
  public StringParameter neighborhood_type=new StringParameter();
  public FloatParameter neighborhood_init=new FloatParameter();
  public LongParameter rand_init=new LongParameter();
  public IntegerParameter iterations_total=new IntegerParameter();
  public IntegerParameter nr_trainvec_total=new IntegerParameter();
  public IntegerParameter vec_normalized=new IntegerParameter();
  public FloatParameter quanterror_map=new FloatParameter();
  public StringParameter url_training_vec=new StringParameter();
  public StringParameter url_training_vec_descr=new StringParameter();
  public StringParameter url_weight_vec=new StringParameter();
  public StringParameter url_quanterr_map=new StringParameter();
  public ListStringParameter url_mapped_input_vec=new ListStringParameter();
  public ListStringParameter url_mapped_input_vec_descr=new ListStringParameter();
  public StringParameter url_unit_descr=new StringParameter();
  public StringParameter description=new StringParameter();

  public Map(String type, int XDim, int YDim, int vec_dim) throws DescriptionException {
    this.type=new StringParameter(type, null);
    this.XDim=new IntegerParameter(XDim, null);
    this.YDim=new IntegerParameter(YDim, null);
    this.vec_dim=new IntegerParameter(vec_dim, null);
    this.unknown = new java.util.Vector();
  }

  public Map(String filename) throws DescriptionException {
    File file = new File(filename);
    if((!file.exists()) || (!file.canRead()))
      throw new DescriptionException("Can't read file: "+file.toString());
    this.filename = filename;

    this.unknown = new java.util.Vector();
    readDescr(file);
  }

  public String endung() {
    return ".map";
  }

  public void writeDescr(String filename) throws DescriptionException {
    File file  = new File(filename);

    try {
      BufferedWriter out = new BufferedWriter(new FileWriter(file));
      out.write(comment.toString());

      out.write("$TYPE"+type.toString()); out.newLine();
      out.write("$XDIM"+XDim.toString()); out.newLine();
      out.write("$YDIM"+YDim.toString()); out.newLine();
      out.write("$VEC_DIM"+vec_dim.toString()); out.newLine();
      if(storage_date.parameter!=null) {
        out.write("$STORAGE_DATE"+storage_date.toString()); out.newLine();
      }
      if(storage_time.parameter!=null) {
        out.write("$STORAGE_TIME"+storage_time.toString()); out.newLine();
      }
      if(training_time.parameter!=-1) {
        out.write("$TRAINING_TIME"+training_time.toString()); out.newLine();
      }
      if(learnrate_type.parameter!=null) {
        out.write("$LEARNRATE_TYPE"+learnrate_type.toString()); out.newLine();
      }
      if(learnrate_init.parameter!=-1) {
        out.write("$LEARNRATE_INIT"+learnrate_init.toString()); out.newLine();
      }
      if(neighborhood_type.parameter!=null) {
        out.write("$NEIGHBORHOOD_TYPE"+neighborhood_type.toString()); out.newLine();
      }
      if(neighborhood_init.parameter!=-1) {
        out.write("$NEIGHBORHOOD_INIT"+neighborhood_init.toString()); out.newLine();
      }
      if(rand_init.parameter!=-1) {
        out.write("$RAND_INIT"+rand_init.toString()); out.newLine();
      }
      if(iterations_total.parameter!=-1) {
        out.write("$ITERATIONS_TOTAL"+iterations_total.toString()); out.newLine();
      }
      if(nr_trainvec_total.parameter!=-1) {
        out.write("$NR_TRAINVEC_TOTAL"+nr_trainvec_total.toString()); out.newLine();
      }
      if(vec_normalized.parameter!=-1) {
        out.write("$VEC_NORMALIZED"+vec_normalized.toString()); out.newLine();
      }
      if(quanterror_map.parameter!=-1) {
        out.write("$QUANTERROR_MAP"+quanterror_map.toString()); out.newLine();
      }
      if(quanterror_map.parameter!=-1  && nr_trainvec_total.parameter!=-1) {
        out.write("$QUANTERROR_VEC "+ quanterr_vec() ); out.newLine();
      }
      if(url_training_vec.parameter!=null) {
        out.write("$URL_TRAINING_VEC"+url_training_vec.toString()); out.newLine();
      }
      if(url_training_vec_descr.parameter!=null) {
        out.write("$URL_TRAINING_VEC_DESCR"+url_training_vec_descr.toString()); out.newLine();
      }
      if(url_weight_vec.parameter!=null) {
        out.write("$URL_WEIGHT_VEC"+url_weight_vec.toString()); out.newLine();
      }
      if(url_quanterr_map.parameter!=null) {
        out.write("$URL_QUANTERR_MAP"+url_quanterr_map.toString()); out.newLine();
      }
      if(url_mapped_input_vec.parameter!=null) {
        out.write("$URL_MAPPED_INPUT_VEC");
        url_mapped_input_vec.writeTo(out);
        out.newLine();
      }
      if(url_mapped_input_vec_descr.parameter!=null) {
        out.write("$URL_MAPPED_INPUT_VEC_DESCR");
        url_mapped_input_vec_descr.writeTo(out);
        out.newLine();
      }
      if(url_unit_descr.parameter!=null) {
        out.write("$URL_UNIT_DESCR"+url_unit_descr.toString()); out.newLine();
      }
      if(description.parameter!=null) {
        out.write("$DESCRIPTION"+description.toString()); out.newLine();
      }
      for (java.util.Enumeration enum = unknown.elements(); enum.hasMoreElements(); ) {
        out.write((String)enum.nextElement()); out.newLine();
      }

      out.close();
    } catch(IOException e) {
      throw new DescriptionException("File communication failure: "+e.getMessage());
    }
  }

  public double quanterr_vec() {
    if(quanterror_map.parameter==-1 || nr_trainvec_total.parameter==-1)
      return -1;      /* Not all necessary parameters available to compute quanterr_vec. */

    return quanterror_map.parameter / (double)nr_trainvec_total.parameter;
  }

  private void readDescr(File source) throws DescriptionException {
    if(!source.exists())
      throw new DescriptionException("Couldn't find file: "+source.toString());

    String str, teil;
    try {
      BufferedReader in = new BufferedReader(new FileReader(source));
      while ((str = in.readLine()) != null) {
        if(str.startsWith("$")) {
          teil=str.substring(1,str.indexOf(" "));

          if(teil.equals("TYPE"))
            type=new StringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("XDIM"))
            XDim=new IntegerParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("YDIM"))
            YDim=new IntegerParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("VEC_DIM"))
            vec_dim=new IntegerParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("STORAGE_DATE"))
            storage_date=new StringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("STORAGE_TIME"))
            storage_time=new StringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("TRAINING_TIME"))
            training_time=new IntegerParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("LEARNRATE_TYPE"))
            learnrate_type=new StringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("LEARNRATE_INIT"))
            learnrate_init=new FloatParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("NEIGHBORHOOD_TYPE"))
            neighborhood_type=new StringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("NEIGHBORHOOD_INIT"))
            neighborhood_init=new FloatParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("RAND_INIT"))
            rand_init=new LongParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("ITERATIONS_TOTAL"))
            iterations_total=new IntegerParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("NR_TRAINVEC_TOTAL"))
            nr_trainvec_total=new IntegerParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("VEC_NORMALIZED"))
            vec_normalized=new IntegerParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("QUANTERROR_VEC")) {} /* the quanterror_vec will be computed on writing */
          else if(teil.equals("QUANTERROR_MAP"))
            quanterror_map=new FloatParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("URL_TRAINING_VEC"))
            url_training_vec=new StringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("URL_TRAINING_VEC_DESCR"))
            url_training_vec_descr=new StringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("URL_WEIGHT_VEC"))
            url_weight_vec=new StringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("URL_QUANTERR_MAP"))
            url_quanterr_map=new StringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("URL_MAPPED_INPUT_VEC"))
            url_mapped_input_vec=new ListStringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("URL_MAPPED_INPUT_VEC_DESCR"))
            url_mapped_input_vec_descr=new ListStringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("URL_UNIT_DESCR"))
            url_unit_descr=new StringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("DESCRIPTION"))
            description=new StringParameter(str.substring(str.indexOf(" ")+1));
          else {
            unknown.add(str);
            System.out.println("Unknown Parameter: "+str);
          }

        } else if(str.startsWith("#")) {
          comment.append(str+"\n");
        } else
          System.out.println("Wrong formatted Map Description File: "+str);
      }
      in.close();
    } catch(IOException e) {
      throw new DescriptionException("Failure communicating with file: "+e.getMessage());
    }

    if((type.parameter==null) || (XDim.parameter==-1) || (YDim.parameter==-1) || (vec_dim.parameter==-1))
      System.out.println("Mandatory Values aren't issued.");
  }

}