package somlib.som.descriptions;

import java.io.*;

public class VectorIn{
/*
 * this class handles file communication with an  Input Vector File
 */

  private java.util.Vector unknown = null;
    /** mandatory Parameters:  **/
  public String type=null;
  public int XDim=-1;
  public int YDim=-1;
  public int vec_dim=-1;
  public float[][] vecs=null;
  public String[] labels = null;

  public VectorIn(String type, int XDim, int vec_dim, float[][] vecs, String[] labels) throws DescriptionException {
    this.type=type;
    this.XDim=XDim;
    this.YDim=1;                    /* this is the only difference to the following constructor */
    this.vec_dim=vec_dim;
    this.vecs=vecs;
    this.labels=labels;
    this.unknown = new java.util.Vector();
    if((XDim*YDim != vecs.length) || ((labels!=null) &&(XDim*YDim != labels.length)))
      throw new DescriptionException("Wrong dimensions given.");
  }

  public VectorIn(String type, int XDim, int YDim, int vec_dim, float[][] vecs, String[] labels) throws DescriptionException {
    this.type=type;
    this.XDim=XDim;
    this.YDim=YDim;
    this.vec_dim=vec_dim;
    this.vecs=vecs;
    this.labels=labels;
    this.unknown = new java.util.Vector();
    if((XDim*YDim != vecs.length) || ((labels!=null) &&(XDim*YDim != labels.length)))
      throw new DescriptionException("Wrong dimensions given.");
  }

  public VectorIn(String filename) throws DescriptionException {
    File file = new File(filename);
    if((!file.exists()) || (!file.canRead()))
      throw new DescriptionException("Can't read file: "+file.toString());

    this.unknown = new java.util.Vector();
    readDescr(file);
  }

  public String endung() {
    return ".in";
  }

  public void writeDescr(String filename) throws DescriptionException {
    File file = new File(filename);

    try {
      BufferedWriter out = new BufferedWriter(new FileWriter(file));

      out.write("$TYPE "+type); out.newLine();
      out.write("$XDIM "+XDim); out.newLine();
      out.write("$YDIM "+YDim); out.newLine();
      out.write("$VEC_DIM "+vec_dim); out.newLine();
      for (java.util.Enumeration enum = unknown.elements(); enum.hasMoreElements(); ) {
        out.write((String)enum.nextElement()); out.newLine();
      }
      for(int i=0; i<XDim*YDim; i++) {
        for(int j=0; j<vec_dim; j++)
          if (vecs[i][j] != 0) {out.write(vecs[i][j]+" ");}
          else {out.write("0 ");}
        out.flush();
        if(labels!=null)
          out.write(labels[i]);
        out.newLine();
        out.flush();

      }
      out.close();
    } catch(IOException e) {
      throw new DescriptionException("File communication failure: "+e.getMessage());
    }
  }

  private void readDescr(File source) throws DescriptionException {
    if(!source.exists() || !source.canRead())
      throw new DescriptionException("Couldn't find file: "+source.toString());

    String str, teil;
    int curvec=0;
    try {
      BufferedReader in = new BufferedReader(new FileReader(source));
      while ((str = in.readLine()) != null) {
        if(str.startsWith("$")) {
          teil=str.substring(1,str.indexOf(" "));

          if(teil.equals("TYPE"))
            type=str.substring(str.indexOf(" ")+1);
          else if(teil.equals("XDIM"))
            XDim=(new Integer(str.substring(str.indexOf(" ")+1))).intValue();
          else if(teil.equals("YDIM"))
            YDim=(new Integer(str.substring(str.indexOf(" ")+1))).intValue();
          else if(teil.equals("VEC_DIM"))
            vec_dim=(new Integer(str.substring(str.indexOf(" ")+1))).intValue();
          else {
            unknown.add(str);
            System.out.println("Unknown Parameter: "+str);
          }

        } else if(!str.startsWith("#") && str.compareTo("") != 0) {
           if(vecs==null) {
             if ((XDim==-1) || (YDim==-1) || (vec_dim==-1))
               throw new DescriptionException("Wrong formatted file.");
             else {
               vecs = new float[XDim*YDim][vec_dim];
               labels = new String[XDim*YDim];
             }
           }
            java.util.StringTokenizer strtok=new java.util.StringTokenizer(str);
            for(int i=0; i<vec_dim; i++) {
              vecs[curvec][i]=(new Float(strtok.nextToken())).floatValue();
            }
            if(strtok.hasMoreTokens())
              labels[curvec]=new String(strtok.nextToken());
            while(strtok.hasMoreTokens())
              labels[curvec]+=" "+strtok.nextToken();
            curvec++;
        }
      }
      if(curvec!=XDim*YDim)
        throw new DescriptionException("Wrong dimensions for map.");
      in.close();
    } catch(IOException e) {
      throw new DescriptionException("Failure communicating with file: "+e.getMessage());
    }

    if((type==null) || (XDim==-1) || (YDim==-1) || (vec_dim==-1) || (vecs==null))
      throw new DescriptionException("Mandatory Values aren't issued.");
  }

}
