package somlib.som.descriptions;

import java.io.*;

public class WeightVector {
/*
 * this class handles file communication for  SOM Weigth Vectors
 */

  private java.util.Vector unknown = null;
  private java.lang.StringBuffer comment = new java.lang.StringBuffer();
    /** mandatory Parameters:  **/
  public StringParameter type=new StringParameter();
  public IntegerParameter XDim=new IntegerParameter();
  public IntegerParameter YDim=new IntegerParameter();
  public IntegerParameter vec_dim=new IntegerParameter();
  public float[][][] vecs=null;
  public String[][] labels = null;


  public WeightVector(String type, int XDim, int YDim, int vec_dim, float[][][] vecs, String[][] labels) throws DescriptionException {
    this.type=new StringParameter(type, null);
    this.XDim=new IntegerParameter(XDim, null);
    this.YDim=new IntegerParameter(YDim, null);
    this.vec_dim=new IntegerParameter(vec_dim, null);
    this.vecs=vecs;
    this.labels=labels;
    this.unknown = new java.util.Vector();
    if((XDim!=vecs.length) || (YDim!=vecs[0].length) || ((labels!=null) && (XDim!=labels.length) && (YDim!=labels[0].length)))
      throw new DescriptionException("Wrong dimensions given.");
  }

  public WeightVector(WeightVector wv, float[][][] vecs, String[][] labels) throws DescriptionException {
    this.type=wv.type;
    this.XDim=wv.XDim;
    this.YDim=wv.YDim;
    this.vec_dim=wv.vec_dim;
    this.vecs=vecs;
    this.labels=labels;
    this.unknown = new java.util.Vector();
    if((XDim.parameter!=vecs.length) || (YDim.parameter!=vecs[0].length) || ((labels!=null) && (XDim.parameter!=labels.length) && (YDim.parameter!=labels[0].length)))
      throw new DescriptionException("Wrong dimensions given.");
  }

  public WeightVector(String filename) throws DescriptionException {
    File file = new File(filename);
    if((!file.exists()) || (!file.canRead()))
      throw new DescriptionException("Can't read file: "+file.toString());

    this.unknown = new java.util.Vector();
    readDescr(file);
  }

  public String endung() {
    return ".wgt";
  }

  public WeightVector update(float[][][] vecs, String[][] labels) {
    this.vecs=vecs;
    this.labels=labels;
    return this;
  }

  public void writeDescr(String filename) throws DescriptionException {
    File file = new File(filename);

    try {
      BufferedWriter out = new BufferedWriter(new FileWriter(file));
      out.write(comment.toString());

      out.write("$TYPE"+type.toString()); out.newLine();
      out.write("$XDIM"+XDim.toString()); out.newLine();
      out.write("$YDIM"+YDim.toString()); out.newLine();
      out.write("$VEC_DIM"+vec_dim.toString()); out.newLine();
      for (java.util.Enumeration enum = unknown.elements(); enum.hasMoreElements(); ) {
        out.write((String)enum.nextElement()); out.newLine();
      }
      for(int y=0; y<YDim.parameter; y++) {
        for(int x=0; x<XDim.parameter; x++) {
          for(int j=0; j<vec_dim.parameter; j++)
            out.write(vecs[x][y][j]+" ");
          out.flush();
          if(labels!=null)
            out.write(labels[x][y]);
          out.newLine();
          out.flush();
        }
      }
      out.close();
    } catch(IOException e) {
      throw new DescriptionException("File communication failure: "+e.getMessage());
    }
  }

  private void readDescr(File source) throws DescriptionException {
    if(!source.exists() || !source.canRead())
      throw new DescriptionException("Couldn't find file: "+source.toString());

    String str, teil;
    int x=0, y=0;
    try {
      BufferedReader in = new BufferedReader(new FileReader(source));
      while ((str = in.readLine()) != null) {
        if(str.startsWith("$")) {
          teil=str.substring(1,str.indexOf(" "));

          if(teil.equals("TYPE"))
            type=new StringParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("XDIM"))
            XDim=new IntegerParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("YDIM"))
            YDim=new IntegerParameter(str.substring(str.indexOf(" ")+1));
          else if(teil.equals("VEC_DIM"))
            vec_dim=new IntegerParameter(str.substring(str.indexOf(" ")+1));
          else {
            unknown.add(str);
            System.out.println("Unknown Parameter: "+str);
          }

        } else if(str.startsWith("#")) {
          comment.append(str+"\n");
        } else {                         //Vector Data
          if(vecs==null) {
            if ((XDim.parameter==-1) || (YDim.parameter==-1) || (vec_dim.parameter==-1))
              throw new DescriptionException("Wrong formatted Weight Vector File.");
            else {
              vecs = new float[XDim.parameter][YDim.parameter][vec_dim.parameter];
              labels = new String[XDim.parameter][YDim.parameter];
            }
          }
          java.util.StringTokenizer strtok=new java.util.StringTokenizer(str);
          for(int i=0; i<vec_dim.parameter; i++) {
            vecs[x][y][i]=(new Double(strtok.nextToken())).floatValue();
          }
          if(strtok.hasMoreTokens())
            labels[x][y]=new String(strtok.nextToken());
          while(strtok.hasMoreTokens())
            labels[x][y]+= " "+strtok.nextToken();
          x++;
          if(x>=XDim.parameter) {
	    x=0;
	    y++;
          }
        }
      }
      if(x!=0 || y!=YDim.parameter)
        throw new DescriptionException("Wrong dimensions for map.");
      in.close();
    } catch(IOException e) {
      throw new DescriptionException("Failure communicating with file: "+e.getMessage());
    }

    if((type.parameter==null) || (XDim.parameter==-1) || (YDim.parameter==-1) || (vec_dim.parameter==-1) || (vecs==null))
      throw new DescriptionException("Mandatory Values aren't issued.");
  }

}
