package somlib.som.labelling;

import java.io.*;

public class LabelSOM {
/*
 * the labels of a node are extracted according to the LabelSOM algorithm
 * and written to a SOM Node Descriptions File
 */

  public static void main(String[] args) throws LabellingException
  /*
   * node-descriptions are read, and written labelled to new (differently named) node-description-files
   */
  {
    if (args.length%2 == 1)
      usage();
    else {
      int param=0, max_lbl=-1;
      float min_wgt=-1;
      String wordfile=null, mapdescr=null, labelled_node_descr=null, labelname=null;
      int verbose=1;

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          usage();

        switch(args[param].charAt(1)) {
          case 'i':
            mapdescr = new String(args[++param]);
            break;
          case 'w':
            wordfile = new String(args[++param]);
            break;
          case 'o':
            labelled_node_descr = new String(args[++param]);
            break;
          case 'l':
            labelname = new String(args[++param]);
            break;
          case 'n':
            max_lbl = Integer.parseInt(args[++param]);
            break;
          case 'm':
            min_wgt = Float.parseFloat(args[++param]);
            break;
          case 'v':
            verbose = Integer.parseInt(args[++param]);
            break;
          default:
            usage();
        }
        param++;
      }

      if(min_wgt==-1 || max_lbl==-1 || wordfile==null || mapdescr==null)
        usage();
      if(labelled_node_descr!= null && labelled_node_descr.indexOf(File.separator) >= 0)
        (new File(labelled_node_descr)).getParentFile().mkdirs();

      LabelSOM labelsom = new LabelSOM((byte)verbose);
      somlib.som.descriptions.Map demap = null;
      somlib.som.descriptions.Node node;
      somlib.som.descriptions.NodeItem ndi;
      somlib.som.descriptions.WeightVector wv;
      somlib.som.descriptions.Err err = null;
      somlib.som.descriptions.Label lbl = null;

      try {
        demap = new somlib.som.descriptions.Map(mapdescr);
        wv = new somlib.som.descriptions.WeightVector(demap.url_weight_vec.parameter);
        if(demap.url_quanterr_map.parameter!=null)
          err = new somlib.som.descriptions.Err(demap.url_quanterr_map.parameter);
        else throw new LabellingException("NWords: SOM Quantisation Error Map needed for LabelSOM.");
        node = new somlib.som.descriptions.Node(demap.url_unit_descr.parameter);
        if(labelname != null)
          lbl = new somlib.som.descriptions.Label(demap.XDim.parameter, demap.YDim.parameter, demap.vec_dim.parameter, min_wgt, max_lbl);
      } catch(somlib.som.descriptions.DescriptionException e) {
        throw new LabellingException("NWords: "+e.getMessage());
      }
      if(labelled_node_descr==null)
        labelled_node_descr=demap.url_unit_descr.parameter;

      java.util.Vector labels;
      String[] words = WordfileReader.loadWords(wordfile, (byte)verbose);
      for(int x=0; x<demap.XDim.parameter; x++) {
        for(int y=0; y<demap.YDim.parameter; y++) {
          labels = new java.util.Vector();
          for(int i=0; i<demap.vec_dim.parameter; i++) {
            labels.add(new somlib.som.descriptions.LabelItem(x, y, words[i], wv.vecs[x][y][i], err.vecs[x][y][i]) );
          }
          labelsom.extractLabels(labels, node.getNodeAt(x, y), lbl, min_wgt, max_lbl);
        }
      }
      try {
        node.writeDescr(labelled_node_descr);
        if(lbl != null)
          lbl.writeDescr(labelname);
      } catch(somlib.som.descriptions.DescriptionException e) {
        throw new LabellingException("LabelSOM: "+e.getMessage());
      }
    }
  }

  static void usage() {
    System.out.println("Usage: somlib.som.labelling.NWords -i <SOM-Map-Description_in> -w <SOM-Template-Vector_in> [-o <SOM-Unit-Description_out>] [-l <SOM-Label-File_out>] -m <minimum weight of label> -n <maximum number of labels> [-v <verbosity>]");

    System.exit(1);
  }


  byte verbose=0;

  public LabelSOM(byte verbose) throws LabellingException
  {
    this.verbose=verbose;
    if (verbose > 0) System.out.println("    LabelSOM initialized.");
  }

  public void extractLabels(java.util.Vector labels, somlib.som.descriptions.NodeItem ndi, somlib.som.descriptions.Label lbl, float min_wgt, int max_lbl) throws LabellingException
  /*
   * the labels of a node are extracted according to the LabelSOM algorithm
   * and the corresponding labels (as initialized in the constructor) are stored in the NodeItem
   */
  {
    if(ndi==null)
      throw new LabellingException("LabelSOM.extractLabels(): Empty node!");
    if (verbose > 1) System.out.println("      extracting Labels for "+ndi.pos_x.parameter+"/"+ndi.pos_y.parameter);

    for(int i=labels.size()-1; i>=0; i--)
      if( ((somlib.som.descriptions.LabelItem)labels.elementAt(i)).wgt < min_wgt )
        labels.removeElementAt(i);

    somlib.som.descriptions.LabelItem sorted[] = (somlib.som.descriptions.LabelItem[])labels.toArray(new somlib.som.descriptions.LabelItem[0]);
    java.util.Arrays.sort(sorted);     // sorts to ascending Quantisation Error

    somlib.som.descriptions.LabelItem lbi;
    if(sorted.length < max_lbl) ndi.nr_unit_labels.parameter = sorted.length;
    else ndi.nr_unit_labels.parameter = max_lbl;
    if(ndi.nr_unit_labels.parameter > 0) {
      ndi.unit_labels.parameter = new String[ndi.nr_unit_labels.parameter];
      ndi.unit_labels_wgt.parameter = new float[ndi.nr_unit_labels.parameter];
      ndi.unit_labels_qe.parameter = new float[ndi.nr_unit_labels.parameter];
    }
    for(int i=0; i<ndi.nr_unit_labels.parameter; i++) {
      lbi = (somlib.som.descriptions.LabelItem)sorted[i];
      if(lbl!=null)
        lbl.addItem(lbi);
      ndi.unit_labels.parameter[i] = lbi.label;
      ndi.unit_labels_wgt.parameter[i] = lbi.wgt;
      ndi.unit_labels_qe.parameter[i] = lbi.qe;
      if (verbose > 1) System.out.println("          "+i+".  extracted "+lbi.label);
    }
  }

}
