package somlib.som.labelling;

import java.io.*;

public class NWords {
/*
 * the labels of the n biggest values of a node-vector are extracted
 * and written to a SOM Node Descriptions File
 */

  public static void main(String[] args) throws LabellingException
  /*
   * node-descriptions are read, and written labelled to new (differently named) node-description-files
   */
  {
    if (args.length%2 == 1)
      usage();
    else {
      int param=0;
      String wordfile=null, mapdescr=null, labelled_node_descr=null, labelname=null;
      int verbose=1, n=-1;

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          usage();

        switch(args[param].charAt(1)) {
          case 'i':
            mapdescr = new String(args[++param]);
            break;
          case 'w':
            wordfile = new String(args[++param]);
            break;
          case 'o':
            labelled_node_descr = new String(args[++param]);
            break;
          case 'l':
            labelname = new String(args[++param]);
            break;
          case 'n':
            n = Integer.parseInt(args[++param]);
            break;
          case 'v':
            verbose = Integer.parseInt(args[++param]);
            break;
          default:
            usage();
        }
        param++;
      }

      if(n==-1 || wordfile==null || mapdescr==null)
        usage();
      if(labelled_node_descr!= null && labelled_node_descr.indexOf(File.separator) >= 0)
        (new File(labelled_node_descr)).getParentFile().mkdirs();

      NWords nwords = new NWords(WordfileReader.loadWords(wordfile, (byte)verbose), (byte)verbose);
      somlib.som.descriptions.Map demap = null;
      somlib.som.descriptions.Node node;
      somlib.som.descriptions.NodeItem ndi;
      somlib.som.descriptions.WeightVector wv;
      somlib.som.descriptions.Err err = null;
      somlib.som.descriptions.Label label = null;

      try {
        demap = new somlib.som.descriptions.Map(mapdescr);
        wv = new somlib.som.descriptions.WeightVector(demap.url_weight_vec.parameter);
        if(demap.url_quanterr_map.parameter!=null)
          err = new somlib.som.descriptions.Err(demap.url_quanterr_map.parameter);
        node = new somlib.som.descriptions.Node(demap.url_unit_descr.parameter);
        if(labelname != null)
          label = new somlib.som.descriptions.Label(demap.XDim.parameter, demap.YDim.parameter, demap.vec_dim.parameter, 0, n);
      } catch(somlib.som.descriptions.DescriptionException e) {
        throw new LabellingException("NWords: "+e.getMessage());
      }
      if(labelled_node_descr==null)
        labelled_node_descr=demap.url_unit_descr.parameter;

      for(int x=0; x<demap.XDim.parameter; x++) {
        for(int y=0; y<demap.YDim.parameter; y++) {
          if( (ndi = node.getNodeAt(x, y))==null)
            throw new LabellingException("NWords: Couldn't get node "+x+"/"+y);
          ndi.nr_unit_labels.parameter = n;
          if(err!=null)
            nwords.giveNWords(ndi, label, wv.vecs[x][y], err.vecs[x][y], n);
          else
            nwords.giveNWords(ndi, label, wv.vecs[x][y], null, n);
        }
      }
      try {
        node.writeDescr(labelled_node_descr);
        if(label != null)
          label.writeDescr(labelname);
      } catch(somlib.som.descriptions.DescriptionException e) {
        throw new LabellingException("NWords: "+e.getMessage());
      }
    }
  }

  static void usage() {
    System.out.println("Usage: somlib.som.labelling.NWords -i <SOM-Map-Description_in> -w <SOM-Template-Vector_in> [-o <SOM-Unit-Description_out>] [-l <SOM-Label-File_out>] -n <number of labels> [-v <verbosity>]");

    System.exit(1);
  }


  String[] words;
  byte verbose=0;

  public NWords(String[] words, byte verbose) throws LabellingException
  /*
   * @words contatins the labels
   */
  {
    this.verbose=verbose;
    if(words==null)
      throw new LabellingException("NWords: No words given.");

    this.words = words;
    if (verbose > 0) System.out.println("    NWords initialized.");
  }

  public void giveNWords(somlib.som.descriptions.NodeItem ndi, somlib.som.descriptions.Label label, float[] values, float[] errs, int n) throws LabellingException
  /*
   * the n biggest values of the @values-array are extracted and the corresponding labels
   * (as initialized in the constructor) are stored in the NodeItem
   */
  {
    if(n > values.length)
      throw new LabellingException("NWords.giveNWords(): More words requested than available.");
    else if(n <= 0)
      throw new LabellingException("NWords.giveNWords(): Can't extract "+n+" words.");
    if (verbose > 1) System.out.println("      giving "+n+" words for "+ndi.pos_x.parameter+"/"+ndi.pos_y.parameter);

    int m=howManyBiggerZero(values); // elias erweiterung. falls alle null sind, dann soll er nicht N labels sondern weniger ausgeben
    if (m<n) n=m;

    if (n==0) ndi.nr_unit_labels.parameter=-1;
    else {
        ndi.nr_unit_labels.parameter=n;
        ndi.unit_labels.parameter = new String[n];
        ndi.unit_labels_wgt.parameter = new float[n];
      if(errs!=null)
        ndi.unit_labels_qe.parameter = new float[n];
      float[] sorted = new float[values.length];
      if (verbose > 1) System.out.println("       sorting.");
      for(int i=0; i<values.length; i++)
        sorted[i]=values[i];
      java.util.Arrays.sort(sorted);
      if (verbose > 1) System.out.println("       done sorting.");
  // changed by elias: want labels to be sorted in output...
  /*    for(int i=0; (act<n) && (i<values.length); i++) {
        if(values[i]>=sorted[values.length-n]) {
          ndi.unit_labels.parameter[act] = words[i];
          ndi.unit_labels_wgt.parameter[act] = values[i];
          if(errs!=null) {
            ndi.unit_labels_qe.parameter[act] = errs[i];
            if(label!=null)
              label.addItem(new somlib.som.descriptions.LabelItem(ndi.pos_x.parameter, ndi.pos_y.parameter, words[i], values[i], errs[i]) );
          } else {
            if(label!=null)
              label.addItem(new somlib.som.descriptions.LabelItem(ndi.pos_x.parameter, ndi.pos_y.parameter, words[i], values[i]) );
          }
          if (verbose > 1) System.out.println("          "+act+".  extracted "+words[i]);
          act++;
        }*/
      int k=-1; // elias: bemerkte, dass wenn values fuer 2 werte gleich sind dann ein problem... so wirds verhindert
      for(int act=0; act<n;act++) {
        if (verbose > 1) System.out.println("       getting word: "+(act+1));
        for(int i=0; i<values.length; i++) {
          if (verbose > 2) System.out.println("       looking at word: "+(i+1)+", out of: "+values.length+" words");
          if((i!=k)&&(values[i]==sorted[values.length-act-1])) {
            k=i;
            if (values[i]==0) {
              if (verbose > 1) System.out.println("           no more good labels for node. (found "+act+")"); // kann ansich nur durch LabelSOM2 passieren (elias)
              act=n;
              break;
            }
            ndi.unit_labels.parameter[act] = words[i];
            ndi.unit_labels_wgt.parameter[act] = values[i];
            if(errs!=null) {
              ndi.unit_labels_qe.parameter[act] = errs[i];
              if(label!=null)
                label.addItem(new somlib.som.descriptions.LabelItem(ndi.pos_x.parameter, ndi.pos_y.parameter, words[i], values[i], errs[i]) );
            } else {
              if(label!=null)
                label.addItem(new somlib.som.descriptions.LabelItem(ndi.pos_x.parameter, ndi.pos_y.parameter, words[i], values[i]) );
            }
            if (verbose > 1) System.out.println("          "+act+".  extracted "+words[i]+", value: "+values[i]);
            break;
          }
        }
      }
    }
  }
// f: faktor um welcher der nachfolgende label kleiner sein muss damit abgebrochen wird
public void giveNWords(somlib.som.descriptions.NodeItem ndi, somlib.som.descriptions.Label label, float[] values, float[] errs, int n, float f) throws LabellingException
  /*
   * the n biggest values of the @values-array are extracted and the corresponding labels
   * (as initialized in the constructor) are stored in the NodeItem
   */
  {
    if(n > values.length)
      throw new LabellingException("NWords.giveNWords(): More words requested than available.");
    else if(n <= 0)
      throw new LabellingException("NWords.giveNWords(): Can't extract "+n+" words.");
    if (verbose > 1) System.out.println("      giving "+n+" words with factor: "+f+" for "+ndi.pos_x.parameter+"/"+ndi.pos_y.parameter);

    int m=howManyBiggerZero(values); // elias erweiterung. falls alle null sind, dann soll er nicht N labels sondern weniger ausgeben
    if (m<n) n=m;

    if (n==0) ndi.nr_unit_labels.parameter=-1;
    else {
      float[] sorted = new float[values.length];
      for(int i=0; i<values.length; i++) sorted[i]=values[i];
      java.util.Arrays.sort(sorted);

      int j=1;
      m=1;
      while ((m<n) && (sorted[values.length-(j+1)]>sorted[values.length-j]*f)) {m++; j++;} // solang wert von nechstem label groesser als bestimter wert mach weiter
      if (m<n) n=m; // nicht alle labels verwenden... scheinbar sind ein paar relativ unwichtige dabei
      if (verbose > 1) System.out.println("        taking: "+n);

      ndi.nr_unit_labels.parameter=n;
      ndi.unit_labels.parameter = new String[n];
      ndi.unit_labels_wgt.parameter = new float[n];
      if(errs!=null) ndi.unit_labels_qe.parameter = new float[n];

      if (verbose > 1) System.out.println("       sorting.");

      if (verbose > 1) System.out.println("       done sorting.");
      int k=-1; // elias: bemerkte, dass wenn values fuer 2 werte gleich sind dann ein problem... so wirds verhindert
      for(int act=0; act<n;act++) {
        if (verbose > 1) System.out.println("       getting word: "+(act+1));
        for(int i=0; i<values.length; i++) {

          if (verbose > 2) System.out.println("       looking at word: "+(i+1)+", out of: "+values.length+" words");
          if((i!=k)&&(values[i]==sorted[values.length-act-1])) {
            k=i;
            if (values[i]==0) {
              if (verbose > 1) System.out.println("           no more good labels for node. (found "+act+")"); // kann ansich nur durch LabelSOM2 passieren (elias)
              act=n;
              break;
            }
            ndi.unit_labels.parameter[act] = words[i];

            ndi.unit_labels_wgt.parameter[act] = values[i];
            if(errs!=null) {
              ndi.unit_labels_qe.parameter[act] = errs[i];
              if(label!=null)
                label.addItem(new somlib.som.descriptions.LabelItem(ndi.pos_x.parameter, ndi.pos_y.parameter, words[i], values[i], errs[i]) );
            } else {
              if(label!=null)
                label.addItem(new somlib.som.descriptions.LabelItem(ndi.pos_x.parameter, ndi.pos_y.parameter, words[i], values[i]) );
            }
            if (verbose > 1) System.out.println("          "+act+".  extracted "+words[i]+", value: "+values[i]);

            i=values.length; // break;
          }
        }
      }
    }
  }

  public static int howManyBiggerZero(float[] v) { // elias... counts how many are bigger than zero (needed for DiffLabelSOM2)
    int m=0;
    for (int i=0;i<v.length;i++) if (v[i]>0) m++;
    return m;
  }
}
