package somlib.som.postprocess;

import java.io.*;

public class HTMLDescr {
/*
 * here a HTML Description of a given map-description (and the corresponding node-descriptions)
 * is produced, to facilitate the interpretation of a SOM's topology
 */

  public static void main(String[] args) throws PostprocessException
  {
    if (args.length%2 == 1)
      usage();
    else {
      int param=0;
      String in=null, out=null, relPath=null;
      boolean qem_som=false, qeav_som=false, params=false, id=false,  qeu=false, qavu=false;
      boolean labels=false, vecs=false, labels_qe=false, labels_wgt=false, vecs_dist=false;
      int verbose=1, exts=0;

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          usage();

        switch(args[param].charAt(1)) {
          case 'i':
            in = new String(args[++param]);
            break;
          case 'o':
            out = new String(args[++param]);
            break;
          case 'r':
            relPath = new String(args[++param]);
            break;
          case 'e':
            exts = Integer.parseInt(args[++param]);
            if(exts<0 || exts>2)
              usage();
            break;
          case 'l':
            if(args[++param].charAt(0) == 't')
              labels=true;
            break;
          case 'm':
            if(args[++param].charAt(0) == 't')
              labels_qe=true;
            break;
          case 'w':
            if(args[++param].charAt(0) == 't')
              labels_wgt=true;
            break;
          case 's':
            if(args[++param].charAt(0) == 't')
              vecs=true;
            break;
          case 'd':
            if(args[++param].charAt(0) == 't')
              vecs_dist=true;
            break;
          case 'u':
            if(args[++param].charAt(0) == 't')
              qeu=true;
            break;
          case 'b':
            if(args[++param].charAt(0) == 't')
              qavu=true;
            break;
          case 't':
            if(args[++param].charAt(0) == 't')
              id=true;
            break;
          case 'p':
            if(args[++param].charAt(0) == 't')
              params=true;
            break;
          case 'q':
            if(args[++param].charAt(0) == 't')
              qem_som=true;
            break;
          case 'a':
            if(args[++param].charAt(0) == 't')
              qeav_som=true;
            break;
          case 'v':
            verbose = Integer.parseInt(args[++param]);
            break;
          default:
            usage();
        }
        param++;
      }

      if(in==null || out==null)
        usage();

      HTMLDescr hdes = new HTMLDescr(in, null, (byte)verbose);
        hdes.qem_som = qem_som;
        hdes.qeav_som = qeav_som;
        hdes.params = params;
        hdes.id = id;
        hdes.qeu = qeu;
        hdes.qavu = qavu;
        hdes.labels = labels;
        hdes.vecs = vecs;
        hdes.labels_qe = labels_qe;
        hdes.labels_wgt = labels_wgt;
        hdes.vecs_dist = vecs_dist;
      hdes.writeHTMLDescr(out, relPath, exts);
    }
  }

  static void usage() {
    System.out.println("Usage: java somlib.som.postprocess.HTMLDescr -i <SOM-Map-Description_in> -o <html_out> [-r <html-relative Path>] [-e <cut # extensions [0-2]>] [-p <show General Parameters [t/f]> ] [-q <show Quantisation Error of SOM [t/f]> ] [-a <show Average Quantisation Error of SOM [t/f]> ] [-t <show ID of Unit [t/f]> ] [-u <show Quantisation Error of Unit [t/f]> ] [-b <show Average Quantisation Error of Unit [t/f]> ] [-l <show Labels [t/f]> ] [-m <show Label Quantisation Error [t/f]> ] [-w <show Label Weight [t/f]> ] [-s <show Mapped Vectors [t/f]> ] [-d <show Vectors Distance [t/f]> ] [-v <verbosity>]");

    System.exit(1);
  }

    /*instance variables*/
  private somlib.textrepresentation.Logfile log = null;
  private byte verbose = 2;
  private somlib.som.descriptions.Map demap;

  public boolean qem_som=true;
  public boolean qeav_som=true;
  public boolean params=true;
  public boolean id=true;
  public boolean qeu=true;
  public boolean qavu=true;
  public boolean labels=true;
  public boolean vecs=true;
  public boolean labels_qe=true;
  public boolean labels_wgt=true;
  public boolean vecs_dist=true;

  public HTMLDescr(String mapdescr, somlib.textrepresentation.Logfile log, byte verbose) throws PostprocessException {
    this.log=log;
    this.verbose=verbose;
    try {
      this.demap = new somlib.som.descriptions.Map(mapdescr);
    } catch(somlib.som.descriptions.DescriptionException e) {
      throw new PostprocessException("HTMLDescr.<init>: couldn't read Map Description: "+e.getMessage());
    }

    if(log!=null)
      log.append("Start: HTMLDescr.");
    if (verbose > 0) System.out.println("  HTMLDescr:");
  }

  public void writeHTMLDescr(String filename, String relPath, int exts) throws PostprocessException {
    if (verbose > 0) System.out.println("    writing HTML-description to "+filename);
    if(relPath==null)
      relPath = new String();
    else if(!relPath.endsWith(File.separator))
      relPath = relPath+File.separator;
    if(exts<0 || exts > 2)
      throw new PostprocessException("Can only cut 0 to 2 extensions.");

    File file = new File(filename);

    try {
      somlib.som.descriptions.Node nd;
      BufferedWriter out = new BufferedWriter(new FileWriter(file));

        /* Kopf */
      out.write("<HTML>"); out.newLine();
      out.write("<HEAD>"); out.newLine();
      out.write("<TITLE>The SOMLib Digital Library Project - "+filename+"</TITLE>"); out.newLine();
      out.write("</HEAD>"); out.newLine();
      out.write("<BODY TEXT=\"#000000\" BGCOLOR=\"#FFFFFF\" LINK=\"#0000EF\" VLINK=\"#55188A\" ALINK=\"#FF0000\">"); out.newLine();
      out.newLine();
      out.write("<CENTER>"); out.newLine();
      out.write("<A HREF=\"http://www.ifs.tuwien.ac.at/ifs\"><IMG SRC=\"images/tu_ifs.gif\" HSPACE=15 VSPACE=4 BORDER=0 HEIGHT=32 WIDTH=55 ALIGN=ABSCENTER></A><B><FONT SIZE=+1>Department of Software Technology</FONT></B><BR>"); out.newLine();
      out.write("<Vienna University of Technology</P>"); out.newLine();
      out.write("</CENTER>"); out.newLine();
      out.write("<HR NOSHADE WIDTH=\"100%\"></P>"); out.newLine();
      out.newLine();
      out.write("<CENTER>"); out.newLine();
      if(demap.description.parameter != null)
        out.write("<H1> "+demap.type.parameter+", "+demap.XDim.parameter+" x "+demap.YDim.parameter+" Self-Organizing Map:  "+demap.description.parameter+" </H1>");
      else
        out.write("<H1> "+demap.type.parameter+", "+demap.XDim.parameter+" x "+demap.YDim.parameter+" Self-Organizing Map </H1>");
      out.newLine();
      if(qem_som) {
        out.write("<H2> Quantization Error of Map: "+(float)demap.quanterror_map.parameter+" </H2>"); out.newLine();
      }
      if(qeav_som) {
        out.write("<H2> Average Quantization Error of Map: "+(float)demap.quanterr_vec()+" </H2>"); out.newLine();
      }
      if(params) {
        out.write("<H3><br>"); out.newLine();
        out.write("  Learnrate: "+demap.learnrate_type.parameter+" with Init: "+demap.learnrate_init.parameter+"  -  Neighborhood: "+demap.neighborhood_type.parameter+" with Init: "+demap.neighborhood_init.parameter+"<br>"); out.newLine();
        if(demap.vec_normalized.parameter>0)
          out.write("  "+demap.iterations_total.parameter+"  Iterations, "+demap.nr_trainvec_total.parameter+" normalized Vectors with "+demap.vec_dim.parameter+" Dimensions<br>");
        else
          out.write("  "+demap.iterations_total.parameter+"  Iterations, "+demap.nr_trainvec_total.parameter+" Vectors with "+demap.vec_dim.parameter+" Dimensions<br>");
        out.newLine();
        out.write("</H3><br>"); out.newLine();
      }
      out.write("</CENTER>"); out.newLine();
      out.newLine();
      out.write("<TABLE BORDER=4 CELLPADDING=4 >"); out.newLine();


        /* Tabelle */
      int index,last;
      java.util.StringTokenizer strtok;
      String curname;
      somlib.som.descriptions.NodeItem ndi;
      try {
        nd = new somlib.som.descriptions.Node(demap.url_unit_descr.parameter);
      } catch(somlib.som.descriptions.DescriptionException e) {
        throw new PostprocessException("HTMLDescr.writeHTMLDescr(): "+e.getMessage());
      }
      for(int y=0; y<demap.YDim.parameter; y++) {
        out.write("<TR>"); out.newLine();
        for(int x=0; x<demap.XDim.parameter; x++) {
          ndi = nd.getNodeAt(x, y);

          out.write("<TD ALIGN=LEFT>"); out.newLine();
          out.write("<A NAME=\"("+x+"/"+y+")\"></A>"); out.newLine();
          if(ndi.unit_id.parameter != null && id) {
            out.write("<CENTER>"+ndi.unit_id.parameter+"</CENTER>"); out.newLine();
          }
          if(ndi.quanterror_unit.parameter>=0 && qeu) {
            out.write("<CENTER>QE_Unit "+(float)ndi.quanterror_unit.parameter +"</CENTER>"); out.newLine();
          }
          if(ndi.quanterror_unit.parameter>=0 && qavu) {
            out.write("<CENTER>QE_Avg_Unit: "+(float)ndi.quanterror_unit_avg() +"</CENTER>"); out.newLine();
          }
          if(id || qeu || qavu) {
            out.write("<hr>"); out.newLine();
          }
          if(ndi.nr_unit_labels.parameter > 0 && labels) {
            for(int i=0; i<ndi.unit_labels.parameter.length; i++) {
              out.write(ndi.unit_labels.parameter[i]);
              if(ndi.unit_labels_qe.parameter!=null && labels_qe)
                out.write(" - QE: "+(float)ndi.unit_labels_qe.parameter[i]);
              if(ndi.unit_labels_wgt.parameter!=null && labels_wgt)
                out.write(" - WGT: "+(float)ndi.unit_labels_wgt.parameter[i]);
              out.write("<br>");
              out.newLine();
            }
            if(vecs) out.write("<br><hr>"); 
            else out.write("<br>"); 
            out.newLine();
          }
          if(ndi.mapped_vecs.parameter != null && vecs) {
            int i=0;
            strtok = new java.util.StringTokenizer(ndi.mapped_vecs.parameter);
            curname = strtok.nextToken();
            index=curname.lastIndexOf(File.separator)+1;
            if(index<0) index=0;
            if(exts==0) last = curname.length();
            else {
              last = curname.lastIndexOf('.');
              if(exts==2 && last>0)  last = curname.lastIndexOf('.', last-1);
              if(last<0)  last = curname.length();
            }
            out.write("<A HREF=\""+relPath+curname.substring(index, last)+"\">"+curname.substring(index, last)+"</A>");
            if(ndi.mapped_vecs_dist.parameter!=null && vecs_dist)
              out.write(" - QE: "+(float)ndi.mapped_vecs_dist.parameter[i]);
            out.newLine();
            while(strtok.hasMoreTokens()) {
              i++;
              curname = strtok.nextToken();
              index=curname.lastIndexOf(File.separator)+1;
              if(index<0) index=0;
              if(exts==0) last = curname.length();
              else {
                last=curname.lastIndexOf('.');
                if(exts==2 && last>0) last = curname.lastIndexOf('.', last-1);
                if(last<0) last=curname.length();
              }
              out.write("<BR><A HREF=\""+relPath+curname.substring(index,last)+"\">"+curname.substring(index, last)+"</A>");
              if(ndi.mapped_vecs_dist.parameter!=null && vecs_dist)
                out.write(" - QE: "+(float)ndi.mapped_vecs_dist.parameter[i]);
              out.newLine();
            }
          }
          out.newLine();
        }
        out.write("</TR>"); out.newLine();
      }

        /* Schluss */
      out.write("</TR>"); out.newLine();
      out.write("</TABLE>"); out.newLine();
      out.newLine();
      out.write("</CENTER>"); out.newLine();
      out.write("<P>"); out.newLine();
      out.newLine();
      out.write("<HR NOSHADE WIDTH=\"100%\"></P>"); out.newLine();
      out.newLine();
      out.write("Comments: <A HREF=\"mailto:rauber@ifs.tuwien.ac.at\">rauber@ifs.tuwien.ac.at</A>"); out.newLine();
      out.write("<BR>"); out.newLine();
      out.write("</P>"); out.newLine();
      out.newLine();
      out.write("</BODY>"); out.newLine();
      out.write("</HTML>"); out.newLine();

      out.close();
    } catch(IOException e) {
      throw new PostprocessException("HTMLDescr.writeHTMLDescr(): File communication failure: "+e.getMessage());
    }
  }

}
