package somlib.som.postprocess;

import java.io.*;

public class LibViewer {
/*
 * here a file of a given map-description (and the corresponding node-descriptions)
 * is produced, that can be read by the LibViewer
 */

  public static void main(String[] args) throws PostprocessException
  {
    if (args.length%2 == 1)
      usage();
    else {
      int param=0;
      String in=null, out=null, suffix = null;
      String imageLocation = null;
      int verbose=1, exts=0;

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          usage();

        switch(args[param].charAt(1)) {
          case 'i':
            in = new String(args[++param]);
            break;
          case 'o':
            out = new String(args[++param]);
            break;
          case 'l':
            imageLocation = new String(args[++param]);
            break;
          case 's':
            suffix = new String(args[++param]);
            break;
          case 'e':
            exts = Integer.parseInt(args[++param]);
            if(exts<0 || exts>2)
              usage();
            break;
          case 'v':
            verbose = Integer.parseInt(args[++param]);
            break;
          default:
            usage();
        }
        param++;
      }

      if(in==null || out==null)
        usage();

      LibViewer libbi = new LibViewer(in, imageLocation, suffix, null, (byte)verbose);
      libbi.writeLibDescr(out, exts);
    }
  }

  static void usage() {
    System.out.println("Usage: java somlib.som.postprocess.LibViewer -i <SOM-Map-Description_in> -o <LibViewer_out> [-l <imageLocation>] [-s <SOM-Vector-Description-Suffix>] [-e <cut # extensions [0-2]>] [-v <verbosity>]");

    System.exit(1);
  }


  private somlib.textrepresentation.Logfile log = null;
  private byte verbose = 0;

  somlib.som.descriptions.Map demap = null;
  String desSuffix, imageLocation;

  public LibViewer(String mapdescr, String imageLocation, String desSuffix, somlib.textrepresentation.Logfile log, byte verbose) throws PostprocessException  {
    this.log=log;
    this.verbose=verbose;
    this.imageLocation = imageLocation;
    if(desSuffix==null) this.desSuffix = somlib.som.descriptions.VectorInDesc.endung();
    else this.desSuffix=desSuffix;
    try {
      this.demap = new somlib.som.descriptions.Map(mapdescr);
    } catch(somlib.som.descriptions.DescriptionException e) {
      throw new PostprocessException("LibViewer.<init>: couldn't read description: "+e.getMessage());
    }

    if(log!=null)
      log.append("Start: LibViewer.");
    if (verbose > 0) System.out.println("  LibViewer:");
  }

  public void writeLibDescr(String filename, int exts) throws PostprocessException {
    if (verbose > 0) System.out.println("    writing LibViewer-description to "+filename);
    if(exts<0 || exts > 2)
      throw new PostprocessException("Can only cut 0 to 2 extensions.");

    somlib.som.descriptions.Node nd;
    somlib.som.descriptions.LibViewer libvi = new somlib.som.descriptions.LibViewer();
    somlib.som.descriptions.VectorInDesc vecInDes;

        /* Tabelle */
      int index,last, i;
      java.util.StringTokenizer strtok;
      String curname;
      somlib.som.descriptions.NodeItem ndi;
      try {
        nd = new somlib.som.descriptions.Node(demap.url_unit_descr.parameter);
      } catch(somlib.som.descriptions.DescriptionException e) {
        throw new PostprocessException("LibViewer.writeLibDescr(): "+e.getMessage());
      }
      if(demap.url_mapped_input_vec_descr.parameter==null)
        if(verbose>1) System.out.println("No location for SOM Input Vector File Descriptions given.");

      for(int y=0; y<demap.YDim.parameter; y++) {
        for(int x=0; x<demap.XDim.parameter; x++) {
          ndi = nd.getNodeAt(x, y);

          if(ndi.mapped_vecs.parameter != null) {
            strtok = new java.util.StringTokenizer(ndi.mapped_vecs.parameter);
            while(strtok.hasMoreTokens()) {
              vecInDes=null;
              curname = strtok.nextToken();
              index=curname.lastIndexOf(File.separator)+1;
              if(index<0) index=0;
              if(exts==0) last = curname.length();
              else {
                last = curname.lastIndexOf('.');
                if(exts==2 && last>0) last = curname.lastIndexOf('.', last-1);
                if(last<0) last=curname.length();
              }
              try {
                if(demap.url_mapped_input_vec_descr.parameter!=null) {
                  i=0;
                  while(i<demap.url_mapped_input_vec_descr.parameter.length && (vecInDes = somlib.som.descriptions.VectorInDesc.fromType(demap.url_mapped_input_vec_descr.parameter[i] + File.separator + curname.substring(index, last) + desSuffix))==null) {
                    if(verbose>0) System.out.println("LibViewer.writeLibDescr(): Couldn't find location for mapped file: "+curname+" at: "+demap.url_mapped_input_vec_descr.parameter[i] + File.separator + curname.substring(index, last) + desSuffix);
                    i++;
                  }
                }
                if(vecInDes != null)
                  libvi.addItem(vecInDes.toLibItem(x, y, imageLocation));
                else
                  libvi.addItem(somlib.som.descriptions.VectorInDesc.standardLibItem(x, y, curname, imageLocation) );
              } catch(somlib.som.descriptions.DescriptionException e) {
                throw new PostprocessException("LibViewer.writeLibDescr(): "+e.getMessage());
              }
            }
          }
        }
      }

    try {
      libvi.writeDescr(filename, demap.url_unit_descr.parameter, demap.XDim.parameter, demap.YDim.parameter);
    } catch(somlib.som.descriptions.DescriptionException e) {
      throw new PostprocessException("LibViewer.writeLibDescr(): "+e.getMessage());
    }
  }

}