package somlib.som.preprocess;

import somlib.som.descriptions.*;

public abstract class ConverterWriter
/*
 * in a Preprocess step, data is first read, converted and finally written
 * therefore any of the writing-steps first converts the data read by a Reader
 * and writes it afterwards;
 * as the file format is the same for all writers, only the converting has to be implemented
 */
{

  public somlib.textrepresentation.Logfile log = null;
  public byte verbose = 2;
  public Reader reader;

  public ConverterWriter(Reader reader, somlib.textrepresentation.Logfile log, byte verbose) throws PreprocessException
  /*
   * the data to be converted and written has been read by a @reader
   */
  {
    this.reader=reader;
    this.log=log;
    this.verbose=verbose;

    if (verbose > 1) System.out.println("    start converting ...");
    convert();
    if (verbose > 1) System.out.println("    done converting.");
  }

  public void writeVecInput(String filename) throws PreprocessException
  /*
   * the writing of the data is done here
   */
  {
    if (verbose > 1) System.out.println("    start writing to "+filename+" ...");

    try {
      VectorIn vc = new VectorIn(reader.type, reader.vecs, reader.dims, reader.iv, reader.labels);
      vc.writeDescr(filename);
    } catch (DescriptionException e) {
      throw new PreprocessException("writeVecInput: "+e.getMessage());
    }
    if (verbose > 1) System.out.println("    writing finished.");
  }

  public abstract void convert() throws PreprocessException;

}