package somlib.som.tools;

import somlib.som.*;
import somlib.som.descriptions.*;
import java.io.*;

public class CreateWeightVectorFromMappedDocuments {
/*
 * elias, 30.12.2000
 * creates a weight vector file (code book vector file) which uses the mapped documents
 * to recalculates the weightvectors as the average of the mapped documents.
 * the idea is to observe the difference between the weight vectors of the map and the
 * average of the documents, which (when using finetuning, or a very flexible map) would
 * be expected to be zero. any varianz might be informative about what a good label might be...
 * so its all about searching a label that would describe the cluster perfectly.
 *
 * this class uses methods i implemented in somlib.som.descriptions.VectorIn
 */

  public static void main(String[] args) throws SOMException {
    if (args.length%2 == 1)
      usage();
    else {
      int param=0;
      String indivVecs=null;
      String nodeDescr=null;
      String outWeightvec=null;
      int verbose=1;

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          usage();

        switch(args[param].charAt(1)) {
          case 'i':
            indivVecs = new String(args[++param]);
            break;
          case 'n':
            nodeDescr = new String(args[++param]);
            break;
          case 'o':
            outWeightvec = new String(args[++param]);
            break;
          case 'v':
            verbose = Integer.parseInt(args[++param]);
            break;
          default:
            usage();
        }
        param++;
      }

      if(indivVecs==null || nodeDescr==null || outWeightvec==null) usage();
      somlib.som.descriptions.Node units = new somlib.som.descriptions.Node(nodeDescr);
      somlib.som.tools.VectorInExtended vi = new somlib.som.tools.VectorInExtended(indivVecs);

      float[][][] codebookvecs = new float[units.XDim.parameter][units.YDim.parameter][vi.vec_dim];
      String[][] codebooklabels = new String[units.XDim.parameter][units.YDim.parameter]; // stores the nodes name eg: (0/0)
      somlib.som.Vector[] vs = null;
      somlib.som.descriptions.NodeItem unit = null;

      if (verbose > 0) System.out.println("  calculating storing weight vector file (as average from documents)");

      somlib.som.Vector zeros = new somlib.som.Vector(vi.vec_dim);

      for(int i=0; i<units.XDim.parameter; i++) {
        for(int j=0; j<units.YDim.parameter; j++) {
          codebooklabels[i][j]="("+i+"/"+j+")";
          unit = units.getNodeAt(i,j);
       System.out.println(i+", "+j);
          String mapped = unit.mapped_vecs.toString();
       if (mapped==null) System.out.println("NULL");
       if (i>0) System.out.println(mapped);
          java.util.StringTokenizer strTok = new java.util.StringTokenizer(mapped," ");
          int toks = strTok.countTokens();
       System.out.println(toks);
          if (toks>0) {
            String[] labels = new String[toks];
            for (int k=0;k<toks;k++) labels[k]=strTok.nextToken();
            vs = vi.getVectorsByLabels(labels);
            codebookvecs[i][j]=calculateAverage(vs);
          } else codebookvecs[i][j]=zeros.v;
        }
      }
      somlib.som.descriptions.WeightVector wc = new somlib.som.descriptions.WeightVector("rect",units.XDim.parameter,units.YDim.parameter,vi.vec_dim,codebookvecs,codebooklabels);
      wc.writeDescr(outWeightvec);
    }
  }

  static void usage() {
    System.out.println("Usage: somlib.som.tools.CreateWeightVectorFromMappedDocuments -i <SOM-Input-Vector_in> -o <SOM-WeightVector_out> -n <SOM-NodeDescription_in> [-v <verbosity>]");
    System.exit(1);
  }

  public void storeWeightsCalculatedAsAveragesOfTheDocuments(String weightFile, String indivFile) throws SOMException
  /*
   * arguments: @weightFile where to store the file created
   *            @indivFile from where to read the individual vectors
   * the get all documents per node, calculate their averages, and create a new weight vector file from these
   * writen by elias 30.12.2000
   * intended to analyze the differences between the codebook and the mean of the documents
   * (this difference might give indications as to how important some words are -> labels)
   */
  {
    try {
      somlib.som.descriptions.VectorIn vi = new somlib.som.descriptions.VectorIn(new String(indivFile));
    } catch(Exception e) {
      throw new SOMException("Map.storeWeightsCalculatedAsAveragesOfTheDocuments: "+e.getMessage());
    }
  }
  // von elias, erstellt am 30.12.2000 benoetigt fuer storeWeightsCalculatedAsAveragesOfTheDocuments
  static float[] calculateAverage(somlib.som.Vector[] vs) throws SOMException {
    try {
      int dims=vs[0].dims; // brauche sowieso mindestens einen vector um einen durchschnitt zu berechnen.
      float[] avg = new float[dims]; // das wird der mittelwert aus allen uebergebenen vectoren
      for (int i=0;i<dims;i++) {
        for (int j=0;j<vs.length;j++) avg[i] += vs[j].v[i];
        avg[i] = avg[i]/vs.length;
      }
      return avg;
    } catch(Exception e) {
      throw new SOMException("Map.storeWeightsCalculatedAsAveragesOfTheDocuments: "+e.getMessage());
    }
  }
}
