package somlib.som.tools;

import somlib.som.labelling.*;
import java.io.*;

public class DiffLabelSOM2 extends NWords {
/*
 * version updated by elias, idea = (1 instead of 2 parameters)
 * this version calculates the difference to the neighbor hood of one unit:
 * sig^p/var-mean(bewertung der nachbarn)
 * bewertung einer unit ist sig^p/var
 *
 * the labels of a node are extracted according to the LabelSOM algorithm
 * and written to a SOM Node Descriptions File
 */

/*
 * the labels of the n biggest values of a node-vector are extracted
 * and written to a SOM Node Descriptions File
 */

  public DiffLabelSOM2(String[] words, byte verbose) throws LabellingException
  /*
   * @words contatins the labels
   */
  {
    super(words,verbose);
  }


  public static void main(String[] args) throws LabellingException
  /*
   * node-descriptions are read, and written labelled to new (differently named) node-description-files
   */
  {
    if (args.length%2 == 1)
      thisUsage();
    else {
      int param=0;
      String wordfile=null, mapdescr=null, labelled_node_descr=null, labelname=null;
      int verbose=1, n=0, p=2;
      float f=0; // factor which determines how many labels to take

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          thisUsage();

        switch(args[param].charAt(1)) {
          case 'i':
            mapdescr = new String(args[++param]);
            break;
          case 'w':
            wordfile = new String(args[++param]);
            break;
          case 'o':
            labelled_node_descr = new String(args[++param]);
            break;
          case 'l':
            labelname = new String(args[++param]);
            break;
          case 'n':
            n = Integer.parseInt(args[++param]);
            break;
          case 'p':
            p = Integer.parseInt(args[++param]);
            break;
          case 'f':
            f = Float.parseFloat(args[++param]);
            break;
          case 'v':
            verbose = Integer.parseInt(args[++param]);
            break;
          default:
            thisUsage();
        }
        param++;
      }

      if(n==0 || wordfile==null || mapdescr==null) thisUsage();
      if ((f>1) || (n==0 && f>0)) thisUsage();

      if(labelled_node_descr!= null && labelled_node_descr.indexOf(File.separator) >= 0)
        (new File(labelled_node_descr)).getParentFile().mkdirs();

      NWords nwords = new NWords(WordfileReader.loadWords(wordfile, (byte)verbose), (byte)verbose);
      somlib.som.descriptions.Map demap = null;
      somlib.som.descriptions.Node node;
      somlib.som.descriptions.NodeItem ndi;
      somlib.som.descriptions.WeightVector wv;
      somlib.som.descriptions.Err err = null;
      somlib.som.descriptions.Label label = null;

      try {
        demap = new somlib.som.descriptions.Map(mapdescr);
        wv = new somlib.som.descriptions.WeightVector(demap.url_weight_vec.parameter);
        if(demap.url_quanterr_map.parameter!=null)
          err = new somlib.som.descriptions.Err(demap.url_quanterr_map.parameter);
        node = new somlib.som.descriptions.Node(demap.url_unit_descr.parameter);
        if(labelname != null)
          label = new somlib.som.descriptions.Label(demap.XDim.parameter, demap.YDim.parameter, demap.vec_dim.parameter, 0, n);
      } catch(somlib.som.descriptions.DescriptionException e) {
        throw new LabellingException("LabelSOM2: "+e.getMessage());
      }
      if(labelled_node_descr==null)
        labelled_node_descr=demap.url_unit_descr.parameter;

      if (err==null) throw new LabellingException("LabelSOM2: no quantisation error file!");

      for(int x=0; x<demap.XDim.parameter; x++) {
        for(int y=0; y<demap.YDim.parameter; y++) {
          if( (ndi = node.getNodeAt(x, y))==null)
            throw new LabellingException("LabelSOM2: Couldn't get node "+x+"/"+y);
          ndi.nr_unit_labels.parameter = n;
          float[] bewertung = new float[demap.vec_dim.parameter];
          for(int i=0; i<demap.vec_dim.parameter; i++) {
            if (err.vecs[x][y][i]!=0)
              bewertung[i]=(float)(Math.pow(wv.vecs[x][y][i],p)/err.vecs[x][y][i]);
            else
              bewertung[i]=0;
          }
          int[][] neighbors = getNeighbors(x,y,demap.XDim.parameter,demap.YDim.parameter);
          if (verbose > 1) System.out.println("    found: "+neighbors.length+" neighbors");
          float[] negBewertung= new float[demap.vec_dim.parameter];
          for (int i=0;i<demap.vec_dim.parameter;i++) negBewertung[i]=0;
          for(int i=0; i<neighbors.length;i++) {
            int x1=neighbors[i][0];
            int y1=neighbors[i][1];

            if (verbose > 1) System.out.println("      "+(i+1)+". neighbor is: ("+x1+", "+y1+")");

            for(int j=0; j<demap.vec_dim.parameter; j++) {
              if (err.vecs[x1][y1][j]!=0)
                negBewertung[j]+=(float)(Math.pow(wv.vecs[x1][y1][j],p)/err.vecs[x1][y1][j]);
              else
                negBewertung[j]+=0;
            }
          }
          if (neighbors.length!=0)
            for (int i=0;i<demap.vec_dim.parameter;i++)
              negBewertung[i]=negBewertung[i]/neighbors.length; // mean
        for (int i=0;i<demap.vec_dim.parameter;i++)
            if (verbose > 1 && x==0 && y==0) System.out.println("      negBewertung: "+i+": "+negBewertung[i]);
          for (int i=0;i<demap.vec_dim.parameter;i++)
            bewertung[i]=bewertung[i]-negBewertung[i]; // difference

          nwords.giveNWords(ndi, label, bewertung, null, n, f);
        }
      }
      try {
        if (verbose > 1) System.out.println("  start writing nodedscr");
        node.writeDescr(labelled_node_descr);
        if (verbose > 1) System.out.println("  done. start writing label");
        if(label != null)
          label.writeDescr(labelname);
      } catch(somlib.som.descriptions.DescriptionException e) {
        throw new LabellingException("LabelSOM2: "+e.getMessage());
      }
      if (verbose > 1) System.out.println("  done all.");
    }
  }

  public static int[][] getNeighbors(int x, int y, int X, int Y) {
    int anzahl=0;
    if (x>0) anzahl+=1;
    if (y>0) anzahl+=1;
    if (x<X-1) anzahl+=1;
    if (y<Y-1) anzahl+=1;
    if ((x>0)&&(y>0)) anzahl+=1;
    if ((x>0)&&(y<Y-1)) anzahl+=1;
    if ((y>0)&&(x<X-1)) anzahl+=1;
    if ((y<Y-1)&&(x<X-1)) anzahl+=1;
    int[][] neighbors=new int[anzahl][2];

    int i=-1;
    if (x>0) {i++; neighbors[i][0]=x-1; neighbors[i][1]=y;}
    if (y>0) {i++; neighbors[i][0]=x; neighbors[i][1]=y-1;}
    if (x<X-1) {i++; neighbors[i][0]=x+1; neighbors[i][1]=y;}
    if (y<Y-1) {i++; neighbors[i][0]=x; neighbors[i][1]=y+1;}
    if ((x>0)&&(y>0)) {i++; neighbors[i][0]=x-1; neighbors[i][1]=y-1;}
    if ((x>0)&&(y<Y-1)) {i++; neighbors[i][1]=x-1; neighbors[i][1]=y+1;}
    if ((y>0)&&(x<X-1)) {i++; neighbors[i][0]=x+1; neighbors[i][1]=y-1;}
    if ((y<Y-1)&&(x<X-1)) {i++; neighbors[i][0]=x+1; neighbors[i][1]=y+1;}
    return neighbors;
  }

  static void thisUsage() {
    System.out.println("Usage: somlib.som.tools.DiffLabelSOM2 -i <SOM-Map-Description_in> -w <SOM-Template-Vector_in> [-o <SOM-Unit-Description_out>] [-l <SOM-Label-File_out>] [-p <power of significanz, default=2>] -n <maximum number of labels> -f <fraction of label 1 which label 2 must reach to be a label> [-p <power of signifikanz, default=2>] [-v <verbosity>]");
// if f = 0 then there is no F effekt
    System.exit(1);
  }

}
