package somlib.som.tools;

import somlib.som.*;
import somlib.som.descriptions.*;
import java.io.*;

public class DiffWeightVectors {
/*
 * elias, 13.01.2001
 *
 * calculates the plain difference between two weightvector files.
 * the idea is to analyze results gained from CreateWeightVectorFromMappedDocuments
 */

  public static void main(String[] args) throws SOMException {
    if (args.length%2 == 1)
      usage();
    else {
      int param=0;
      String weightVec1=null,weightVec2=null;
      String outWeightVec=null;
      int verbose=1;

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          usage();

        switch(args[param].charAt(1)) {
          case 'a':
            weightVec1 = new String(args[++param]);
            break;
          case 'b':
            weightVec2 = new String(args[++param]);
            break;
          case 'o':
            outWeightVec = new String(args[++param]);
            break;
          case 'v':
            verbose = Integer.parseInt(args[++param]);
            break;
          default:
            usage();
        }
        param++;
      }

      if(weightVec1==null || weightVec2==null || outWeightVec==null) usage();

      somlib.som.descriptions.WeightVector wv1 = new somlib.som.descriptions.WeightVector(weightVec1);
      somlib.som.descriptions.WeightVector wv2 = new somlib.som.descriptions.WeightVector(weightVec2);

      if (verbose > 1) {
        System.out.println("1 xDim: "+wv1.XDim.parameter);
        System.out.println("1 yDim: "+wv1.YDim.parameter);
        System.out.println("1 dims: "+wv1.vec_dim.parameter);
        System.out.println("1 type: "+wv1.type.parameter);

        System.out.println("2 xDim: "+wv2.XDim.parameter);
        System.out.println("2 yDim: "+wv2.YDim.parameter);
        System.out.println("2 dims: "+wv2.vec_dim.parameter);
        System.out.println("2 type: "+wv2.type.parameter);
      }
      if (wv1.vec_dim.parameter!=wv2.vec_dim.parameter || wv1.XDim.parameter!=wv2.XDim.parameter || wv1.YDim.parameter!=wv2.YDim.parameter || wv1.type.parameter.compareTo(wv2.type.parameter)!=0)
        throw new SOMException("WeightVectors not same type or size.");

      float[][][] v=new float[wv1.XDim.parameter][wv1.YDim.parameter][wv1.vec_dim.parameter];
      for(int x=0; x<wv1.XDim.parameter; x++)
        for(int y=0; y<wv1.YDim.parameter; y++)
          for(int j=0; j<wv1.vec_dim.parameter; j++)
            v[x][y][j]=wv1.vecs[x][y][j]-wv2.vecs[x][y][j];

      somlib.som.descriptions.WeightVector wvOut = new somlib.som.descriptions.WeightVector(wv1.type.parameter,wv1.XDim.parameter,wv1.YDim.parameter,wv1.vec_dim.parameter,v,wv1.labels);
      wvOut.writeDescr(outWeightVec);
    }
  }

  static void usage() {
    System.out.println("Usage: somlib.som.tools.DiffWeightVectors -a <WeightVec 1> -b <WeightVec 2> <-o WeightVec Out> [-v <verbosity>]");
    System.exit(1);
  }
}
