package somlib.textrepresentation;

import somlib.textrepresentation.preprocess.email.*;
import java.util.*;
import java.io.*;
import java.lang.*;

public class emailexc {

  public static void main(String[] args) {
    if (args.length%2 == 1)
      usage();
    else {
      int param=0;
      String input_mailfile=null;
      String output_mailfile=null;
      String output_emails=null;
      String descrDest = null;
      int verbose=2;

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          usage();

        switch(args[param].charAt(1)) {
          case 'i':
            input_mailfile = new String(args[++param]);
            break;
          case 'o':
            output_mailfile = new String(args[++param]);
            break;
          case 'u':
            output_emails = new String(args[++param]);
            break;
          case 'r':    //description-directory
            descrDest = new String(args[++param]);
            break;
          case 'v':
            verbose = (int)args[++param].charAt(0)-(int)'0';
            break;
          default:
            usage();
        }
        param++;
      }

      if(output_mailfile==null || output_emails==null)
        usage();

      if(input_mailfile!=null)
        mailfile(input_mailfile, output_mailfile,verbose);
      emails(output_mailfile, output_emails, descrDest, verbose);
    }
  } //main

  static void usage() {
    System.out.println("Usage: somlib.textrepresentation.preprocess.email.emailexc -i <input_mailfile> -o <partitioned_emails> -u <plain_emails> [-r <vector-description-dir>] [-v <verbosity>]");

    System.exit(1);
  }


  private static void mailfile(String input_mailfile, String output_mailfile, int verb)
  {
    File indir1 = null;
    if (input_mailfile.compareTo("") != 0) {
      indir1 = new File(input_mailfile);
      if (!indir1.exists()) { // dir existiert nicht
        System.out.println("Mailfile input file does not exist.");
        System.exit(-1);
      }
    } else {
      System.out.println("Mailfile input file missing.");
        System.exit(-1);
    }
    File outdir1 = null;
    if (output_mailfile.compareTo("") != 0) {
      outdir1 = new File(output_mailfile);
      if (!outdir1.exists()) 
      {
        if(verb>0) System.out.println("Creating directory: "+output_mailfile);
        try {
          outdir1.mkdirs();
        } catch (Exception e) 
        {
            System.out.println("Cannot create directory.");
            System.exit(-1);
        }
      }
    } else {
      System.out.println("Mailfile output directory missing.");
      System.exit(-1);
    }

    try {
      MailFile mf = new MailFile(input_mailfile);
      mf.extractEmails((byte) verb, null, output_mailfile,".email");
    } catch (Exception e) {
      System.out.println(e.getMessage());
      System.exit(-1);
    }
  } //mailfile()


  private static void emails(String input_emails, String output_emails, String descrDir, int verb) {
    File indir2 = null;
    if (input_emails.compareTo("") != 0) {
      indir2 = new File(input_emails);
      if (!indir2.exists()) {
        System.out.println("EMail input directory does not exist.");
        System.exit(-1);
      }
    } else {
      System.out.println("EMail input directory missing.");
      System.exit(-1);
    }
    File outdir2 = null;
    if (output_emails.compareTo("") != 0) {
      outdir2 = new File(output_emails);
      if (!outdir2.exists()) {
        if(verb>0) System.out.println("Creating directory: "+output_emails);
        try {
          outdir2.mkdirs();
        } catch (Exception e) 
        {
          System.out.println("Cannot create directory.");
          System.exit(-1);
        }
      }
    } else {
      System.out.println("EMails output directory missing.");
      System.exit(-1);
    }
    File outdir3 = null;
    if (descrDir!=null && descrDir.compareTo("") != 0) {
      outdir3 = new File(descrDir);
      if (!outdir3.exists()) {
        if(verb>0) System.out.println("Creating directory: "+descrDir);
        try {
          outdir3.mkdirs();
        } catch (Exception e) {
          System.out.println("Cannot create directory.");
          System.exit(-1);
        }
      }
    }

    try {
      File f0 = new File(input_emails);
      File[] fili0 = f0.listFiles();
      EMails em = new EMails(fili0);
      if(descrDir != null)
        em.extractPlainBodies((byte)verb,null,"",".plain",".vec", output_emails, descrDir);  // wenn die extension abgeschnitten werden soll, einfach statt ""  ".email" eintragen

      else
        em.extractPlainBodies((byte)verb,null,".email",".plain",".vec", output_emails, null);
    } catch (Exception e) {
      System.out.println(e.getMessage());
      System.exit(-1);
    }
  }  //emails()

}
