package somlib.textrepresentation.parser;

import java.io.*;
import java.lang.*;
import java.util.Hashtable;
import somlib.textrepresentation.*;

/**
 * "this word"-> "thi", "his", "is ", "s w", " wo", "wor", "ord".
 *
 * @author elan
 * @version 1.0 (22.08.1999)
 */
public class NGrams extends Object implements Parser {
  private int n;
  private boolean numbers;
  private boolean spaces;

/**
 * @param n words size
 * @param numbers if true then include numbers (else ignore)
 * @param spaces if true include spaces (else ignore)
 */
  public NGrams(int n, boolean numbers, boolean spaces) {
    this.n = n;
    this.numbers = numbers;
    this.spaces = spaces;
  } // constructor

/**
 * Does all the work.
 *
 * @param files to be used to create the individual vectors, the prefix of the files will be used again for the the individual vectors
 * @param outDir where the individual vectors are to be stored
 * @param verbose extra information is given when set to true
 * @param override determines if files should be replaced or skipped
 * @param fileSuffix of the output files
*/
  public File[] createFiles (File[] files ,String outDir, byte verbose,Logfile log, boolean override, String fileSuffix) throws TextrepresentationException
  {
    if (files == null) {throw new TextrepresentationException("NGrams: createFiles: filelist not defined.");}
    if (outDir == null) {throw new TextrepresentationException("NGrams: createFiles: output directory not defined."); }

    String myLine = null; // readLine aud input datei wird dort gespeichert
    StringBuffer tmpWord = new StringBuffer(); // wort wird dort gespeichert bevor in die liste aufgenommen

    File[] modifiedFiles = new File[files.length];
    int index = 0;
    Hashtable idtf = new Hashtable();
    File dirOutF = null;


    if (verbose>0) System.out.println("Start NGrams.");
    try {
      File outFile;
      BufferedReader in;
      ObjectOutputStream out;
      dirOutF = new File(outDir);
      if (dirOutF.mkdirs()) {if (verbose>1) System.out.println("Directory created: "+dirOutF.toString());}


    if (verbose>1) System.out.println("  Files: "+files.length);

      for (int i = 0; i < files.length; i++) {
        if (!files[i].isDirectory()) {

          // output datei anlegen
          outFile = new File(outDir + dirOutF.separator +files[i].getName()+ fileSuffix);
          if (override || outFile.createNewFile())
          {

          if (verbose>2) System.out.println("    "+files[i].toString());

            modifiedFiles[index] = outFile; index++;
            in = new BufferedReader(new FileReader(files[i]));
            out = new ObjectOutputStream(new FileOutputStream(outFile));

            idtf.clear();
            boolean inWord;
            boolean inLine;
            while ((myLine = in.readLine()) != null) {
              if (myLine.length() >= n) {
                int l=0;
                inLine=true;
                while (inLine) {
                  int w=0;
                  inWord=true;
                  tmpWord.delete(0,tmpWord.length());
                  int u=0;
                  while (inWord) {
                    char c = myLine.charAt(l+w+u);
                    if  ( (Character.isLetter(c)) || (numbers && Character.isDigit(c)) || (spaces && Character.isWhitespace(c)) ) {
                      w++;
                      tmpWord.append(c);
                      if (w >= n) { // eintragen in liste
                        inWord=false;
                        String str = new String(tmpWord);
                        Integer n = (Integer) idtf.get(str.toLowerCase());
                        if (n != null) {
                          idtf.remove(str.toLowerCase()); // habe keine methode gefunden mit der man den value zu einem key aendern kann
                          idtf.put(str.toLowerCase(), new Integer(n.intValue()+1)); // erhoehe den zaehler um eins
                        }
                        else {idtf.put(str.toLowerCase(), new Integer(1));}
                      }
                    } // c is wanted
                    else {u++;}
                    if (l+u+w >= myLine.length()) {inWord=false; inLine=false;}
                  } // inWord
                  l++;
                } //inLine
              } // zeile lang genug
            } // solange nicht EOF

            out.writeObject(idtf);
            out.flush(); out.close();
            in.close();
          } // file existiert schon -> ignorieren, mit naechstem weitermachen
        }
      } // gehe alle dateien in der liste durch
      File[] modifiedFilesCut = new File[index];
      for (int i = 0; i<index; i++) modifiedFilesCut[i] = modifiedFiles[i];
      if (verbose>0) System.out.println("Done.");
      return modifiedFilesCut;
    } catch (Exception e) {throw new TextrepresentationException("Error in Words: createFiles: " + e.getMessage());}
  } // createFiles

} // class words
