package somlib.textrepresentation.preprocess.bookmarks;

import java.util.*;
import java.io.*;
import somlib.textrepresentation.*;

//den titel und content gesondert behandeln?! zb doppelt drucken?

public class HTlminator {

  private BufferedReader page=null;
  private PrintWriter dest = null;
  private StringBuffer purified;
  private Logfile log_;
  private byte verbose;

    /** for testing reasons only */
/*  public static void main(String[] args) {
         //parsed file "text" und schreibt in file "leer":
    HTlminator test;
    try {
      test = new HTlminator((byte)4,null,"text", "leer");
    } catch(BookmarksException e) {
      System.err.println(e.getMessage());
    }
  }*/

  public HTlminator(byte verbose, Logfile l, String input, String output, String descrput) 
                   throws BookmarksException
  /*
    constructor - the main work is done by the method extract(),
    everything that's done here is the dirty work: initializing, cleaning, ...
  */
  {
    log_=l;
    this.verbose=verbose;

    if (verbose > 2)
      System.out.println("    HTlminator: parsing "+input+" to "+output);

    try {
      page=new BufferedReader(new FileReader(input));
      dest = new PrintWriter(new FileWriter(output, false));   //ueberschreibt vorhandenes file
    } catch(FileNotFoundException e) {
      if (verbose > 1) System.out.println("Internal Fault getting the correct names of the pages: "+e.getMessage());
      throw new BookmarksException("HTlminator: Internal Fault getting the correct names of the pages: "+e.getMessage());
    } catch(IOException e) {
      if (verbose > 1) System.out.println("Couldn't convert pages: "+e.getMessage());
      throw new BookmarksException("HTlminator: Couldn't convert pages: "+e.getMessage());
    }
    purified = new StringBuffer();

    try {
      extract();
    } catch(IOException e) {
      if (verbose > 1) System.out.println("Couldn't parse "+input+" to "+output+", "+e.getMessage());
      throw new BookmarksException("HTlminator: Couldn't parse "+input+" to "+output+", "+e.getMessage());
    }

    dest.print(purified.toString());
    dest.flush();
    dest.close();
    try {
      page.close();
    } catch(IOException e) {
      if (verbose > 1) System.out.println("Couldn't close input: "+e.getMessage());
      throw new BookmarksException("HTlminator: Couldn't close input: "+e.getMessage());
    }

    if(descrput != null) {
      try {
        new VecDescrWriter(verbose, l, (new File(input)).lastModified(), input, descrput);
      } catch(BookmarksException e) {
        if (verbose > 1) System.out.println("Couldn't create vector description.files: "+e.getMessage());
        throw new BookmarksException("VecDescrWriter: Couldn't create vector description files: "+e.getMessage());
      }
    }

    if (verbose > 2)
      System.out.println("    HTlminator: successfully finished parsing.");
  }

  private void extract() throws IOException
  /*
    the html-pages content is parsed by using a stringtokenizer.
    anytime a meta-construct is found '<', it is extinguished.
    also special characters, that start with an embassent '&' are cleaned
    from the text. if possible, it is replaced by the base letter, as for
    example any letter with an accent or >umlaut< is replaced by the letter
    without the accent.
  */
  {
    String str = " ", line = null;
    StringTokenizer parts = null;
    char previous=' '; //gebraucht, um zu berprfen, ob ein '&...'  ein sonderzeichen ist
    int i=0, mode=0;  //i als hilfsvariable, 
                     //mode als "speicher", wenn in einem construkt ein zeilenvorschub ist

    while ((line=page.readLine()) != null) {
      parts = new StringTokenizer(line.trim(), "<>&", true);
      while(parts.hasMoreTokens()) {
        if(str.length()>0)
          previous = str.charAt(str.length()-1);
        str = parts.nextToken().trim();
         if((str.startsWith("<")) || (mode==1)) {     //html-meta-construct .. loeschen
          mode=1;
          while(parts.hasMoreTokens()) {
            if (parts.nextToken().startsWith(">")) {
              mode=0;
              break;
            }
          }
        } else if(str.startsWith(">")) {    //zeichen teil vom text
          //ist aber unwichtig, also passiert nichts
        } else if((!str.startsWith("&")) && (mode==0)) {  //das ist der gute fall
          //writePurified(" "+str);
         purified.append(" "+str);
        } else {       //str.startsWith("&")
          if(parts.hasMoreTokens())
            str = parts.nextToken();
          else str="";
          i = str.indexOf(";");
          if((i > 0) && (i+1<str.length())) {   //Sonderzeichen!
            char current = str.charAt(i+1);
            if(((previous>='a' && previous<='z') || (previous>='A' && previous<='Z') || (previous=='\t')) &&
              ((current>='a' && current<='z') || (current>='A' && current<='Z') || (current=='\t'))) {
              purified.append(str.charAt(0));   //war ein Umlaut oder so
            } else
              purified.append(" ");
            purified.append(str.substring(i+1));
          } else if (i>0) {  //Sonderzeichen, aber nachher ist das token aus, also wars ein Formatzeichen
            //Textformatierungszeichen brauch ma nicht
          } else        //war ein ganz normales '&'
            purified.append(" "+str);
        }
      }
    }
  } //extract

} //class

