package somlib.textrepresentation.preprocess.bookmarks;

import java.util.*;
import java.io.*;
import java.lang.*;
import somlib.textrepresentation.*;

public class getNparse {
  private static String BKMKSUFFIX=".bkmk";

  public static void main(String[] args) {
    if (args.length%2 == 1) {
      usage();
    } else {
      int param=0;
      getNparse starter;
      String file =null;
      String dir = null;
      String bufferDir = null;
      String descrDest = null;
      int mode = 2, verbose = 2;

      while(param < args.length) {      //getting the command-line parameters
        if((args[param].length()>2) || (args[param].charAt(0)!='-'))
          usage();

        switch(args[param].charAt(1)) {
          case 'f':    //bookmark File
            file = new String(args[++param]);
            break;
          case 'd':    //ziel Directory
            dir = new String(args[++param]);
            break;
          case 'm':    //Modus
            mode = (int)args[++param].charAt(0)-(int)'0';
            break;
          case 'b':    //Buffer-directory
            bufferDir = new String(args[++param]);
            break;
          case 'r':    //description-directory
            descrDest = new String(args[++param]);
            break;
          case 'v':
            verbose = (int)args[++param].charAt(0)-(int)'0';
            break;
          default:
            usage();
        }
        param++;
      }

      if(dir==null || bufferDir==null || (file==null && mode!=4))
        usage();

      try {
        starter = new getNparse((byte)verbose,null,file,bufferDir,dir,".bkmk", descrDest, ".vec", mode);
      } catch(BookmarksException e) {
        System.err.println("Error: "+e.getMessage());
      }
    }
  } //main

  static void usage() {
    System.out.println("Usage: java somlib.textrepresentation.preprocess.bookmarks.getNparse [-f <bookmark-file>] -d <destination-dir> -b <buffer-dir> -r <vector-description-dir> -m <mode> [-v <verbosity>]");
    System.out.println("mode 1: clear <destination-dir> and create files newly");
    System.out.println("mode 2: get all and overwrite present files");
    System.out.println("mode 3: add new files, skip present files");
    System.out.println("mode 4: use present files only");
    System.exit(1);
  }


  private Logfile log_;
  private byte verbose;

  public getNparse(byte verbose, Logfile l, String bookmarkFile, String bufferDirname, String dirname,
                   String parsedSuffix, String descrDest, String descrSuffix, int mode) throws BookmarksException
  /*
    the main work of the class is done here in the constructor:
    it coordinates the parsing of the bookmark-file (bookmarkFile), 
    getting of the pages to a buffer directory (bufferDirname)
    and parsing the original pages to a destination-directory (dirname) with a suffix (parsedSuffix),
    vector description files will be stored in the destination-directory (descrDest) with the suffix (descrSuffix)
  */
  {
    log_=l;
    this.verbose=verbose;
    Runtime rt = Runtime.getRuntime();

    if (verbose > 0) System.out.println("  getNparse: entered...");
    if(log_ != null) {
      log_.append("Start: Bookmarks Module.");
      log_.append("  Params:");
      log_.append("    bookmarkFile: "+bookmarkFile);
      log_.append("    buffer-directory: "+bufferDirname);
      log_.append("    destination-directory: "+dirname);
      log_.append("    parsed-suffix: "+parsedSuffix);
      log_.append("    description-destination-directory: "+descrDest);
      log_.append("    description-suffix: "+descrSuffix);
      log_.append("    mode: "+mode);
      log_.append("  Params end.");
    }

    if(mode<1 || mode>4)
    {
      if (verbose > 1) System.out.println("Unknown mode: "+mode);
      throw new BookmarksException("getNParse: Unknown mode: "+mode);
    }

    prepareDir(dirname, mode);     //check whether directory is available
    prepareDir(bufferDirname, mode);
    if(descrDest!=null)
      prepareDir(descrDest, mode);

    if(mode!=4)  //mode==4 won't get it newly, what is done here
    {
        ///***  getting the urls  ****///
      URLextr getter = null;
      try {
        getter = new URLextr(verbose,log_,bookmarkFile);
      } catch(FileNotFoundException e) {
        if (verbose > 1) System.out.println("The bookmarkFile is not available: "+bookmarkFile+", "+e.getMessage());
        throw new BookmarksException("getNParse: The bookmarkFile is not available: "+bookmarkFile+", "+e.getMessage());
      }


      String destFilename;
      File destFile;
      Process process;
      URLitem url;

      while((url=getter.getURLs()) != null)
      {
        destFilename = getNparse.createFilename(url, bufferDirname);
        destFile = new File(destFilename);

        System.out.println("Copying "+url.url+" to "+destFile.toString());
        if(mode==1 || mode==2 || !destFile.exists())  //conditions for getting a page
        {
          try {   //get the pages by executing wget on each entry in the bookmarks
            process = rt.exec("wget -nv -nd -O "+destFile.toString()+" -F "+url.url);
          } catch (IOException e) {
            if (verbose > 1) System.out.println("Couldn't execute command: "+e.getMessage());
            throw new BookmarksException("getNparse: Couldn't execute command: "+e.getMessage());
          }

          try {
            BufferedWriter out = new BufferedWriter(new FileWriter(destFile));
            out.write("<!URL "+url.url+">");
            out.newLine();
          }catch(IOException e) {
            if (verbose > 1) System.out.println("Couldn't append url: "+e.getMessage());
            throw new BookmarksException("getNparse: Couldn't append url: "+e.getMessage());
          }
        }

      }  //while
      if (verbose > 1) System.out.print("  getNparse: got files needed from web ...");
    }  //mode!=4
    else if (verbose > 1) System.out.print("  getNparse: don't need to get files ...");


       ///***  parsing the files in dir(name)  ****///
    File bufferDir = new File(bufferDirname);
    File destination, descr;
    String[] files = bufferDir.list();
    HTlminator converter;
    String destinationName;

    if (verbose > 1) System.out.println(" starting to parse files in: "+bufferDirname);
    for(int i=files.length-1;i>=0;i--)
    {
      destinationName = dirname+File.separator+files[i]+parsedSuffix;
      destination = new File(destinationName);
      if(mode==1 || mode==2 || !destination.exists())         //conditions for getting a page
      {
        try {
          if(descrDest != null) {
            descr = new File(descrDest+File.separator+files[i]+descrSuffix);
            if(mode==1 || mode==2 || !descr.exists())
              converter = new HTlminator(verbose,log_,bufferDir.getPath()+File.separator+files[i],destination.toString(), descr.toString());
          } else
            converter = new HTlminator(verbose,log_,bufferDir.getPath()+File.separator+files[i],destination.toString(), null);
        } catch(BookmarksException e) {
          throw new BookmarksException("getNparse: "+e.getMessage());
        }
      } else if (verbose>2)
        System.out.println("    "+destination.toString()+" already exists");
    }

    if (verbose > 0) System.out.println("  getNparse: finished.");
  }

  public static String createFilename(URLitem url, String dir)
  /*
    get's a url and label of the original page and returns the name of the file its content will be stored in
  */
  {
    if(!(new File(dir+File.separator+url.name+BKMKSUFFIX)).exists())
      return dir+File.separator+url.name+BKMKSUFFIX;

    int index = 0;
    while( (new File(dir+File.separator+url.name+index+BKMKSUFFIX)).exists() )
      index++;

    return dir+File.separator+url.name+index+BKMKSUFFIX;
  }

  public static void prepareDir(String dirname, int mode) throws BookmarksException
  /*
    prepares the directory dirname appropriate to mode
    to put it differently, checks whether the directory is writable, to be created,...
  */
  {
    File dir = new File(dirname);

    if (dir.exists()) {
      if(!(dir.isDirectory() && dir.canWrite()))
        throw new BookmarksException("getNparse: A file or not writable directory called "+dirname+" already exists!");
      else if(mode == 1) {   // && dir.isDirectory() && dir.canWrite()
        if(!getNparse.emptyDir(dir))
          throw new BookmarksException("prepareDir: Couldn't clear directory: "+dirname);
      }
    } else
      dir.mkdirs();
  }

  public static boolean emptyDir(File dir)
    //clears an entire directory
  {
    if(!dir.isDirectory() || !dir.canWrite())
      return false;

    String[] files = dir.list();
    File toKill;

    for(int i=files.length-1;i>=0;i--) {
      toKill = new File(dir.getPath()+File.separator+files[i]);
      if(!toKill.delete())
        return false;
    }

    return true;
  }

} //class
