package somlib.textrepresentation.preprocess.email;

import java.io.*;
import java.lang.*;
import somlib.textrepresentation.preprocess.email.EMailException;
import somlib.textrepresentation.preprocess.email.contentType.*;

/**
 * Gets header information from an email file.
 * Parses header for certain fields.
 *
 * @author Elias
 * @version 1.0 (18.08.1999)
 */
public class EMailHeader extends Object {

  private String subject;
  private String author;
  private String date;
  private String disposition = new String();
  private long size=0;
  private int priority = 3;
      /* die priority skala ist wie folgt:
        X-Priority: 1 (Highest)
        X-Priority: 2 (High)
        X-Priority: 3 (Normal)   //ist default => wenn nicht vorhanden wird diese stufe angenommen
        X-Priority: 4 (Low)
        X-Priority: 5 (Lowest) */
  private ContentType contentType = null;
  public String helpContentType = "";

/**
 * Opens file, parses header and sets values.
 *
 * @param filename the email file to be parsed
 * @exception somlib.parser.preperation.email.EMailException
 */
  public EMailHeader(File filename) throws EMailException {
    try {
      String str1 = null;

      BufferedReader in = new BufferedReader(new FileReader(filename));

      // solang bis eine leerzeile kommt (oder EOF)
      while ((str1 = in.readLine()).compareTo("") != 0) {
        if (str1.toUpperCase().startsWith("SUBJECT:")) {subject = str1.substring(9,str1.length());}
        else if (str1.toUpperCase().startsWith("FROM:")) {author = str1.substring(6, str1.length());}
        else if (str1.toUpperCase().startsWith("DATE:")) {date = str1.substring(6, str1.length());}
        else if (str1.toUpperCase().startsWith("CONTENT-TYPE:") && helpContentType==null) {helpContentType = str1.substring(14,str1.length()).toUpperCase();}
        else if (str1.toUpperCase().startsWith("CONTENT-LENGTH:")) {size=Long.parseLong(str1.substring(16, str1.length()).trim());}
        else if (str1.toUpperCase().startsWith("CONTENT-DISPOSITION:")) {
          int pos;
          if((pos=str1.indexOf(";")) < 0)
            disposition = str1.substring(21, str1.length()).trim();
          else
            disposition = str1.substring(21, pos).trim();
        }
        else if (str1.toUpperCase().startsWith("X-PRIORITY:")) {
            priority = (int)str1.charAt(12)-(int)'0';
        }
      }
     // es besteht die moeglichkeit, informationen die in helpContentType stehen an ContentType zu uebergeben. bei plain text aber nicht notwendig.
      if (subject == null) {subject = "";}
      if (helpContentType.compareTo("") == 0) {contentType = new PlainText();}
      else if (helpContentType.compareTo("TEXT") == 0) {contentType = new PlainText();}
      else if (helpContentType.indexOf("TEXT/PLAIN") != -1) {contentType = new PlainText();}
      in.close();
    } catch (Exception e) {System.out.println("EMailHeader: bad file: " + filename + ". Check whether Mailfile is mbox type."); throw new EMailException("EMailHeader: "+e.getMessage());}
  }  // constructor

/*
 * @return subject of the email, if the content-type is not declared an empty string is returned
 */
  public String subject() {
    if(contentType != null)
      return subject;

    return new String();
  }

/*
 * @return date of the email, if the content-type is not declared an empty string is returned
 */
  public String date() {
    if(contentType != null)
      return date;

    return new String();
  }

/*
 * @return author of the email, if the content-type is not declared an empty string is returned
 */
  public String author() {
    if(contentType != null)
      return author;

    return new String();
  }

/*
 * @return size of the email
 */
  public long size() {
      return size;
  }

/*
 * @return priority of the email
 */
  public long priority() {
      return priority;
  }

/*
 * @return disposition of email, if the content-type is not declared an empty string is returned
 */
  public String disposition() {
    if(contentType != null)
      return disposition;

    return new String();
  }

/*
 * @return content-type of the email, if the content-type is not declared,  'null' is returned
 */
  public ContentType contentType() {
      return contentType;
  }

/*
 * @return description of the email, if the content-type is not declared,  'null' is returned
 */
  public String getDescription() {
    if(contentType != null)
      return contentType.getDescription();

    return new String();
  }

} // EMailHeader
