package somlib.textrepresentation.preprocess.email;

import java.io.*;
import java.lang.*;
import java.util.*;
import somlib.textrepresentation.*;

/**
*
* @author Elias
* @version 1.0 (20.08.1999)
*/
public class MailFile extends Object {

// kleinste unmoegliche ID. (+1 ist die erste moegliche)
  private final int CminID = -1;

  private String mailfile;

// in dieser classe koennte man spaeter noch unterscheidungen zw. mbox, MH,..
// einfueheren. sowie das ganze remote machen. zunaechst wird aber nur ein
// lokales file verwendet, im mbox format.

/**
 *  This Class is used to extract emails out of a mailfile in mbox format.
 *
 *  @param mailfile the full path and name
 *
 */
  public MailFile(String mailfile) {
    this.mailfile=mailfile;
  } // constructor

/**
 *  Saves every email in the mailfile as single file
 *  the mailfile must be in "mbox" format. in "mbox" format emails are in one file and are seperated by a line starting with "From ".
 *
 *  @param verbose if true then extra information is given
 *  @param directory where the email files are to be stored
 *  @param suffix for example "1234.email" where 1234 is the ID und ".email" the suffix (the dot must be included)
 *  @exception EMailException in every error case (file read/write,..)
 *  @return list of files which were created
 */
// zeichen weise aus dem emailFile einlesen. speziell fuer mbox format:
// suchen nach "From " (case sensitiv) am anfang einer zeile.
// neue email files anlegen.
  public File[] extractEmails(byte verbose, Logfile l, String directory, String suffix) throws EMailException {
    File dir = new File(directory);
    String fileSysPathSeparator = new String(dir.separator);
    String[] fileList = null;

    File[] modifiedFiles = null;
    int index = 0;

    if (verbose > 0) System.out.println("  Mailfile: start extracting..."); // debug
    if (l != null) {
      l.append("Start: Mailfile Module.");
      l.append("  Params:");
      l.append("    directory: "+directory);
      l.append("    suffix: "+suffix);
      l.append("    mailfile: "+mailfile);
      l.append("  Params end.");
    }

    try { // verzeichnis: falls nicht vorhanden wird neues angelegt
      if (dir.mkdir()) {if (verbose > 1) System.out.println("    Verzeichnis angelegt: " + dir.toString() + ".");}
    } catch (Exception e) {throw new EMailException("MailFile: Can't mkdir: " + dir.toString() + e.getMessage());}

    try { // liste aller dateien im verzeichnis
      fileList = dir.list();
      if (fileList == null) { // fehler aufgetreten
        throw new IOException("MailFile: Can't get files in " + dir + ".");}
    } catch (Exception e) {throw new EMailException("MailFile: May not access " + dir.toString() + ": " + e.getMessage());}

    // start fileID bestimmen
    int ID = getMaxFileID(fileList, suffix) + 1;

    Hashtable modFiles = new Hashtable();

    // mailfile parsen, neue files anlegen
    try {
      BufferedReader in = new BufferedReader(new FileReader(mailfile));
      BufferedWriter out = null;
      String str = null;

      if (verbose > 2) System.out.println("      Reading Lines from Mailfile:"); // debug

      while ((str = in.readLine()) != null) { // not EOF

        if (verbose > 2) System.out.println("      : "+str); // debug

        if ((str.length() > 4) && (str.substring(0,5).compareTo("From ") == 0)) { // out buffer wechseln (neue email)
          if (out != null) {out.flush(); out.close();}
          File outfile = new File(directory + fileSysPathSeparator + ID + suffix);
          modFiles.put(new Integer(index),outfile); index++;
          if (!outfile.createNewFile()) {throw new EMailException("Internal Error!");} // die datei sollte eigentlich nicht existieren
          out = new BufferedWriter(new FileWriter(outfile));
          ID++; // fuer den naechsten
        }
        out.write(str); out.newLine();
      } // while not EOF
      in. close();
      out.flush(); out.close();
      if (verbose > 1) System.out.println("    Files Created. Max ID: " + ID); // debug
    } catch (Exception e) {throw new EMailException("MailFile: " + e.getMessage() +  ".");}

    modifiedFiles = new File[modFiles.size()];
    for (int i=0;i<modFiles.size();i++) {modifiedFiles[i] = (File) modFiles.get(new Integer(i));}
    if (verbose > 0) System.out.println("  Mailfile: end.");

    if (l!=null)
      l.append("End: Mailfile Module.");

    return modifiedFiles;
  } // extractEmails

/**
 *  Sucht im uebergegebenen Verzeichnis die hoechste verwendete ID.
 *  @param dir ort wo die emails landen sollen
 *  @exception IOException fehler beim lesen der Dateien
 */
  private int getMaxFileID(String[] fileList, String endung) throws EMailException{
    StringBuffer str = new StringBuffer("");
    int max = CminID;

    if (fileList.length != 0) { // bestimmt aus liste von strings den mit der groessten nummer, wobei endungen zu beachten sind
      for (int i = 0; i < (fileList.length); i++) {
        try {
          str.delete(0,str.length());
          str.append(fileList[i]);
          if ((str.length() > endung.length()) && (str.substring(str.length()-endung.length(), str.length()).toLowerCase().compareTo(endung.toLowerCase()) == 0)) {
            // die letzten zeichen des filenamens entsprechen der const "endung"
            try {
              Integer j = new Integer(str.substring(0, str.length()-endung.length()));
              if (max < j.intValue()) {max = j.intValue();}
            } catch (Exception e) {} // wenn keine zahl einfach ignorieren
          }
        } catch (Exception e) {throw new EMailException("MailFile: getMaxFileID: Internal Error: " + e.getMessage());}
      }
    return max;
    }
    else {return CminID;}; // wird nur erreicht wenn keine files im verzeichnis
  } // getMaxFileID

} // class MailFile



