package somlib.textrepresentation.preprocess.email;

import java.util.*;
import java.io.*;
import java.lang.*;
import somlib.textrepresentation.*;

public class VecDescrWriter {

  private EMailHeader h;
  private BufferedWriter dest = null;
  private String filename;
  private Logfile log_;
  private byte verbose;

  public VecDescrWriter(byte verbose, Logfile l, EMailHeader h, long size,  String output) throws EMailException {
    log_=l;
    this.verbose=verbose;
    this.h = h;
    this.filename = output;

    if (verbose > 2)
      System.out.println("      VecDescrWriter: writing Vector-Description to "+output);

    try {
      dest = new BufferedWriter(new FileWriter(output, false));   //ueberschreibt vorhandenes file
    } catch(FileNotFoundException e) {
      if (verbose > 1) System.out.println("Internal Fault getting the correct names of the pages: "+e.getMessage());
      throw new EMailException("VecDescrWriter: Internal Fault getting the correct names of the pages: "+e.getMessage());
    } catch(IOException e) {
      if (verbose > 1) System.out.println("Couldn't create Vector-Description-File: "+e.getMessage());
      throw new EMailException("VecDescrWriter: Couldn't create Vector-Description-File: "+e.getMessage());
    }

    writeInfo(size);

    try {
      dest.close();
    } catch(IOException e) {
      if (verbose > 1) System.out.println("Couldn't close input: "+e.getMessage());
      throw new EMailException("VecDescrWriter: Couldn't close input: "+e.getMessage());
    }

    if (verbose > 2)
      System.out.println("      VecDescrWriter: successfully finished writing.");
  }


  private void writeInfo(long size) throws EMailException {
    try {
      dest.write("$VEC_ID "+filename.substring(1+filename.lastIndexOf(File.separator)) ); dest.newLine();
      dest.write("$TYPE email"); dest.newLine();
      dest.write("$MAILTYPE "+h.contentType().contentTypeName()); dest.newLine();
      dest.write("$SUBJECT "+h.subject()); dest.newLine();
      String address = h.author();
      int pos;
      if((pos=address.indexOf("("))>=0) {
        dest.write("$AUTHOR "+address.substring(pos+1, address.indexOf(")")-1) ); dest.newLine();
      } else if((pos=address.indexOf("<")) < 0) {
        pos = address.indexOf("@");
        dest.write("$AUTHOR "+address.substring(0,pos)); dest.newLine();
      } else if(address.indexOf('"') < 0) {
        dest.write("$AUTHOR "+address.substring(0,pos).trim()); dest.newLine();
      } else {
        String addtmp = address.substring(0,pos).trim();
        dest.write("$AUTHOR "+addtmp.substring(1,addtmp.length()-1)); dest.newLine();
      }
      dest.write("$URL "+address); dest.newLine();
      dest.write("$DOMAIN "+extractDomain(address)); dest.newLine();

      java.util.Date currentTime = new java.util.Date();
      java.text.SimpleDateFormat formatter = new java.text.SimpleDateFormat ("dd.MM.yyyy");
      dest.write("$DATE "+ formatter.format(currentTime)); dest.newLine();
      formatter = new java.text.SimpleDateFormat ("hh:mm:ss");
      dest.write("$TIME "+ formatter.format(currentTime)); dest.newLine();
      dest.write("$TIMEWRITTEN "+h.date() ); dest.newLine();

      dest.write("$SIZE "+size); dest.newLine();
      dest.write("$READ f"); dest.newLine();
      dest.write("$PRIORITY "+h.priority()); dest.newLine();
      dest.write("$ANSWERED f"); dest.newLine();
      if(h.disposition().toUpperCase().equals("ATTACHMENT")) {
        dest.write("$ATTACHMENT t"); dest.newLine();
      } else {
        dest.write("$ATTACHMENT f"); dest.newLine();
      }
      dest.write("$DESCRIPTION "+h.getDescription()); dest.newLine();
    } catch(IOException e) {
      try {
        dest.close();
      } catch(IOException ex) {}
      if (verbose > 1) System.out.println("Couldn't write descr: "+e.getMessage());
      throw new EMailException("VecDescrWriter: Couldn't write descr: "+e.getMessage());
    }
  }

  private String extractDomain(String url) {
    int index = url.indexOf("@");
    if(index > 0) {
      index = url.indexOf(">", index);
      if(index < 0) index = url.indexOf(" ", url.indexOf("@") );
      if(index < 0) return url.substring(url.lastIndexOf('.', url.length())+1, url.length()).toLowerCase();
      else return url.substring(url.lastIndexOf('.', index)+1, index).toLowerCase();
    }
    return "void";
  }

}