package somlib.textrepresentation.preprocess.email.contentType;

import java.io.*;
import somlib.textrepresentation.preprocess.email.*;

/**
* Handles plain text emails which are the simplest type of emails.
*
* @author elias
* @version 1.0 (18.08.1999)
*/
public class PlainText extends Object implements ContentType {

  public String description = new String();
  private static final int DESC_LEN = 255;

/**
 *  Gets the email text by filtering the header.
 *  Writes the subject and the body to a text file.
 *  @param inFileName email file to parse.
 *  @param outFile Destination file for the body of the email.
 *  @param subject Subject of the email, it is written in to the first line of the output file.
 *  @exception EMailException If any error occurs (file read or write).
 *  @returns the size of the body as long
 */
  public long getBodyText(File inFile, File outFile, String subject) throws EMailException {
    String str = null;
    BufferedReader in = null;
    BufferedWriter out = null;
    try {
      in = new BufferedReader(new FileReader(inFile));
      out = new BufferedWriter(new FileWriter(outFile));
      out.write(subject); out.newLine();
      // skipHeader
      while (in.readLine().compareTo("") != 0) {}
      // read rest
      while ((str = in.readLine()) != null) {
         out.write(str); out.newLine();
        if(description.length() < DESC_LEN) {
          description = description + " "+str;
          if(description.length() >= DESC_LEN)
            description = description.substring(0, DESC_LEN);
        }
      }
      out.flush(); out.close();
      in.close();
      return outFile.length();
    } catch (Exception e) {System.out.println("Error in PlainTextEMailImpl."); throw new EMailException("PlainTextEMailImpl: "+e.getMessage());}
  }

  public String contentTypeName() {
    return new String("TEXT");
  }

  public String getDescription() {
    return description;
  }

}
